#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Scott Hendrickson"
__license__="Simplified BSD"

import unittest
from acscsv import *  

class TestAcsCSV(unittest.TestCase):
    """Unit tests ofr common CSV utility functions"""
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testCleanField(self):
        a = AcsCSV("|", False)
        self.assertEquals(a.cleanField("laksjflasjdfl;a"), "laksjflasjdfl;a")
        self.assertEquals(a.cleanField("\r\n \n\r \r\r \n\n"), "")
        self.assertEquals(a.cleanField("\r\na \n\r \r\r a\n\n"), "a       a")
        self.assertEquals(a.cleanField("asdf|asdf,,adsf|asdf"), "asdf asdf,,adsf asdf")
        b = AcsCSV(",", False)
        self.assertEquals(b.cleanField("asdf|asdf,,adsf|asdf"), "asdf|asdf  adsf|asdf")
        self.assertEquals(b.cleanField(245), "245")
        self.assertEquals(b.cleanField(a), INTERNAL_EMPTY_FIELD)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Scott Hendrickson"
__license__="Simplified BSD"
import sys
import json as json_formatter
try:
    import ujson as json
except ImportError:
    try:
        import json
    except ImportError:
        import simplejson as json

LIST_ITEMS = "list-items"

def reflect_json(d):
    return json.loads(walk_label(d))

def walk_label(x, label=None):
    if type(x) == type({}):
        for field in x:
            if label is None:
                new_field = field
            else:
                new_field = "%s:%s"%(label, field)
            x[field] = walk_label(x[field], label=new_field)
    elif type(x) == type([]):
        if len(x) > 0:
            a = x[0]
            x = [ walk_label(a, label="%s_%s"%(label, LIST_ITEMS))  ]
        else:
            x = [ "%s_%s"%(label, LIST_ITEMS) ]
    else:
        return label
    if label is None:
        return json.dumps(x)
    else:
        return x



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Josh Montague"
__license__="Simplified BSD"

import sys
import json
import acscsv

# for custom twitter output, import both the fields module and the code module
from twitter_acs_fields import *
import twitter_acs


#
# define any custom field extractor classes here. 
#



class CustomCSV( twitter_acs.TwacsCSV ):
    """
    Test class for experimenting with new output combinations. This class should inherit
    from the appropriate module.class in the core library. Compliance and invalid records 
    are handled by the parent class' procRecord() method. This class should only define a 
    new get_output_list() method which overrides the parent method and determines the 
    custom output. 
    """       
    
    def get_output_list(self, d):
        """
        Use this method to overwrite the output method in the parent class. Append values from 
        desired payload fields to the output list, which is returned at the end of this method. 
        Non-str values (ints, lists, etc), must be cast as str or will return 'None'. 
        
        Take a JSON Activity Streams payload as a Python dictionary. Extract the fields 
        appended to output_list and then return output_list. 
        """
        output_list = [] 

        # twitter country code
        output_list.append( Field_location_twitter_country_code(d).value )

        # matching rules -- note the str cast; value is a list 
        output_list.append( str(Field_gnip_rules(d).value) )

        # username 
        output_list.append( Field_actor_preferredusername(d).value )

        # geo-tag coords (returns a list, cast to str)
        output_list.append( str(Field_geo_coordinates(d).value) )


        # done building output list 
        return output_list 



if __name__ == "__main__":
    """
    Receive data from stdin (decompressed JSON-lines only) and process using the existing 
    code, but let this module's get_output_list() method define the output fields. 
    """

    # Get the appropriate object by mocking the constructor in the main gnacs.py code. most 
    #   common command-line options (flags) don't matter since we're explicitly defining the 
    #   fields to be printed in the method above 
    processing_obj = CustomCSV("|", None, *[True]*7) 

    for line_number, record in processing_obj.file_reader(): 
        # note: this doesn't handle broken pipe errors  
        sys.stdout.write( u"{}\n".format( processing_obj.procRecord(record, emptyField="None") ))




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Josh Montague"
__license__="MIT License"

import sys
import unittest
from StringIO import StringIO
from stocktwits_acs import *


# valid activity from source (eg from data/) 
VALID_ACTIVITY = """{"id":"tag:gnip.stocktwits.com:2012:note/24443686","verb":"post","body":"Today&#39;s w/l $txmd $nuro $achn $royl $useg $dgaz $grpn $live $oncy $camt","actor":{"id":"person:stocktwits:358559","objectType":"person","displayName":"Relentless","preferredUsername":"Relentlessiam","followersCount":63,"followingCount":40,"followingStocksCount":0,"statusesCount":632,"summary":null,"links":[{"href":null,"rel":"me"}],"link":"http://stocktwits.com/Relentlessiam","image":"http://avatars.stocktwits.net/production/358559/thumb-1401411068.png","tradingStrategy":{"approach":"Momentum","holdingPeriod":"Day Trader","experience":null,"assetsFrequentlyTraded":["Equities","Options"]},"classification":[]},"object":{"id":"note:stocktwits:24443686","objectType":"note","postedTime":"2014-07-08T12:03:04Z","updatedTime":"2014-07-08T12:03:04Z","summary":"Today&#39;s w/l $txmd $nuro $achn $royl $useg $dgaz $grpn $live $oncy $camt","link":"http://stocktwits.com/Relentlessiam/message/24443686"},"provider":{"displayName":"StockTwits","link":"http://stocktwits.com"},"link":"http://stocktwits.com/Relentlessiam/message/24443686","entities":{"stocks":[{"displayName":"Achillion Pharmaceuticals, Inc.","stocktwits_id":718,"symbol":"ACHN","exchange":"NASDAQ","industry":"Biotechnology","sector":"Healthcare","trending":false},{"displayName":"Camtek Ltd.","stocktwits_id":1192,"symbol":"CAMT","exchange":"NASDAQ","industry":"Semiconductor - Integrated Circuits","sector":"Technology","trending":false},{"displayName":"LiveDeal, Inc.","stocktwits_id":2508,"symbol":"LIVE","exchange":"NASDAQ","industry":"Internet Information Providers","sector":"Technology","trending":false},{"displayName":"NeuroMetrix Inc.","stocktwits_id":2920,"symbol":"NURO","exchange":"NASDAQ","industry":"Medical Instruments & Supplies","sector":"Healthcare","trending":false},{"displayName":"Oncolytics Biotech Inc.","stocktwits_id":2983,"symbol":"ONCY","exchange":"NASDAQ","industry":"Biotechnology","sector":"Healthcare","trending":false},{"displayName":"Royale Energy Inc.","stocktwits_id":3414,"symbol":"ROYL","exchange":"NASDAQ","industry":"Independent Oil & Gas","sector":"Basic Materials","trending":false},{"displayName":"US Energy Corp.","stocktwits_id":3971,"symbol":"USEG","exchange":"NASDAQ","industry":"Independent Oil & Gas","sector":"Basic Materials","trending":false},{"displayName":"Groupon","stocktwits_id":8404,"symbol":"GRPN","exchange":"NASDAQ","industry":"Internet Information Providers","sector":"Technology","trending":true},{"displayName":"VelocityShares 3x Inverse Natural Gas ETN","stocktwits_id":11298,"symbol":"DGAZ","exchange":"NYSEArca","industry":"Exchange Traded Fund","sector":"Financial","trending":false},{"displayName":"TherapeuticsMD Inc.","stocktwits_id":11596,"symbol":"TXMD","exchange":"NYSEMkt","industry":null,"sector":null,"trending":false}],"chart":null,"sentiment":null,"video":null},"gnip":{"language":{"value":"en"}}}\r\n"""


class Teststocktwits(unittest.TestCase):
    """Unit tests of common CSV utility functions"""

    def setUp(self):
        """
        Stocktwits (Activity Streams) constructor takes five args - create one example of each for 
        use in all tests (loop over the self.objs list).
        """ 

        self.delim = '|'

        # use a dict to make it easier to refer to the particular cases
        self.objs = { 
                    "base": StacsCSV(self.delim, None, False, False, False)
                    , "user": StacsCSV(self.delim, None, True, False, False) 
                    , "struct": StacsCSV(self.delim, None, False, True, False) 
                    , "influence": StacsCSV(self.delim, None, False, True, False) 
                    , "all": StacsCSV(self.delim, True, True, True, False) 
                    , "keypath": StacsCSV(self.delim, False, False, False, "source:title") 
                    }

        ## set some vars once, update when needed
        self.base_length = 3
        # count of extra fields added by each of these options
        self.keypath_length = 1
        self.user_length = 3 
        self.struct_length = 3 
        self.influence_length = 3 
        self.all_length = self.base_length \
                            + self.user_length \
                            + self.influence_length \
                            + self.struct_length 


    def tearDown(self):
        """
        """
        pass

    
    #
    # helpful to group test methods that are related into sections
    #
    
    def test_reader(self):
        """Test that our file_reader method is working as expected."""
        # grab the base instance 
        o = self.objs["base"]
        
        # without eg a for loop, we use the generator's next() method 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 


    def test_sample_data(self):
        """
        Check that we can use each test object's procRecord method on each record in the 
        stocktwits_native_sample.json example file without raising an Exception.
        """
        # get a temporary object
        tmp = self.objs["base"]

        # grab the correct data file 
        datafile = "./data/stocktwits_sample.json"

        # loop over all test stocktwits processing objects
        for o in self.objs.values():
            # loop over records in test file 
            for i, record in o.file_reader(datafile):
                # if there's a problem parsing, this method will raise an Exception
                record_string = o.procRecord(record)


    def test_base_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the base instance 
        o = self.objs["base"]

        # use sample record in this module 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            # procRecord returns a delimited string
            record_string = o.procRecord(record)
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length 
                            )  
         

    def test_user_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["user"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.user_length 
                            ) 

    def test_influence_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["influence"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.influence_length 
                            ) 


    def test_all_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["all"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            loop = True
            record_string = o.procRecord(record)
            # should have many extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.all_length 
                            )  



if __name__ == "__main__":
    unittest.main()



# -*- coding: UTF-8 -*-
__author__="Scott Hendrickson, Josh Montague"
__license__="Simplified BSD"

import sys
from .twitter_acs_fields import *
from .acscsv import *

class TwacsCSV(AcsCSV):
    """Subset of Twitter fields with specified delimiter.  Please see help for options"""

    def __init__(self
                , delim
                , options_keypath
                , options_geo
                , options_user
                , options_rules
                , options_urls
                , options_lang
                , options_influence
                , options_struct
                ):
        super(TwacsCSV, self).__init__(delim, options_keypath)
        self.options_geo = options_geo 
        self.options_user = options_user
        self.options_rules = options_rules
        self.options_urls = options_urls
        self.options_lang = options_lang
        self.options_influence = options_influence
        self.options_struct = options_struct
        # pre-create all of teh objects needed for parsing
        #for name, obj in inspect.getmembers(sys.modules[__name__]):
        #    if name.startswith("Field_"):
        #        setattr(self, name.lower()+"_", obj(None))

    def procRecordToList(self, d):
        """
        Take a JSON Activity Streams payload as a Python dictionary. Check activity for system 
        information and compliance handling. If necessary, return the system info or compliance 
        message. Otherwise, if the activity is valid, return the result of calling the 
        appropriate output() method.  
        """
        record = []
        try:
            verb = Field_verb(d).value
            # see: http://support.gnip.com/apis/consuming_streaming_data.html#Consuming 
            system_msgs = ["error", "warning", "info"]
            if verb in system_msgs: 
                msg = "Unidentified meta message"
                for mtype in system_msgs:
                    if mtype in d:
                        if "message" in d[mtype]:
                            msg = d[mtype]["message"]
                        elif "message" in d:
                            msg = d["message"]
                        continue
                    mtype = "Unidentified"
                record.append('-'.join([gnipRemove, mtype]))
                record.append(gnipDateTime)
                record.append(msg)
                return record
            elif verb == "delete":
                record.append(d["object"]["id"])
                record.append(gnipDateTime)
                record.append('-'.join([gnipRemove, verb]))
                return record
            elif verb == "scrub_geo":
                record.append(d["actor"]["id"])
                record.append(gnipDateTime)
                record.append('-'.join([gnipRemove, verb]))
                return record
        except KeyError:
            record.append(gnipError)
            record.append(gnipRemove)
            return record

        # at this point, verb is an acceptable record 
        return self.get_output_list(d) 


    def get_output_list(self, d):
        """
        Take a JSON Activity Streams payload as a Python dictionary. Specify the particular 
        output fields (and their order) by constructing and returning a list of the 
        desired extractor values. Default values for missing fields are set in the _Field class 
        and can be overridden.  
        """
        output_list = [] 

        # base output = id | timestamp | body
        output_list.append( Field_id(d).value )
        output_list.append( Field_postedtime(d).value )
        output_list.append( Field_body(d).value )

        # urls 
        if self.options_urls:
            #
            # TODO: add back this exception handling for -x option
            # https://github.com/DrSkippy/Gnacs/blob/16dd146fb05d02d7c1e3f282254e6718fd13303f/acscsv/twacscsv.py#L97 
            #
            # gnip 
            val = Field_gnip_urls(d).value
            if isinstance(val, list): 
                try:
                    output_list.append( self.buildListString( [ x["expanded_url"] for x in val ] ) )   
                except KeyError:
                    output_list.append(Field_gnip_urls(d).default_value)
            else: 
                output_list.append( val )  
            # twitter
            val = Field_twitter_entities_urls(d).value  
            if isinstance(val, list):
                url_list = self.buildListString( [ x["url"] for x in val ] ) 
                exp_url_list = self.buildListString( [ x["expanded_url"] for x in val ] ) 
            else:
                url_list = val
                exp_url_list = val 
            output_list.append( url_list )  
            output_list.append( exp_url_list )  
    
        # languages 
        if self.options_lang:
            # actor
            #   - this field has *very* infrequently contained unicode chars. drop them.
            output_list.append( Field_actor_language(d).value.encode('ascii', 'ignore') ) 
            # classifications
            output_list.append( Field_gnip_language_value(d).value ) 
            output_list.append( Field_twitter_lang(d).value ) 
    
        # rules
        if self.options_rules:
            val = Field_gnip_rules(d).value 
            if isinstance(val, list):
                # output: '[" value (tag)", ... ]'
                output_list.append( 
                    self.buildListString( 
                        [ "{} ({})".format( x["value"], x["tag"] ) for x in Field_gnip_rules(d).value ]
                        )
                    ) 
            else: 
                output_list.append( val )  

        # geo-related fields
        if self.options_geo:
            # geo-tag 
            val = Field_geo_coordinates(d).value
            # keep self.geoCoordsList for backward compatibility
            self.geoCoordsList = None
            if isinstance(val, list):
                output_list.append( str(val) ) 
                self.geoCoordsList = val 
            else:
                output_list.append( val ) 
            output_list.append( Field_geo_type(d).value )
            val = Field_location_geo_coordinates(d).value 
            if isinstance(val, list): 
                output_list.append( str(val) )  
            else:
                output_list.append( val )  
            output_list.append( Field_location_geo_type(d).value )
            output_list.append( Field_location_displayname(d).value )  
            output_list.append( Field_location_twitter_country_code(d).value )  
            # user  
            output_list.append( Field_actor_utcoffset(d).value )  
            output_list.append( Field_actor_location_displayname(d).value )  
            # profileLocations
            output_list.append( Field_gnip_profilelocations_displayname(d).value )  
            output_list.append( Field_gnip_profilelocations_objecttype(d).value )  
            output_list.append( Field_gnip_profilelocations_address_country(d).value )  
            output_list.append( Field_gnip_profilelocations_address_region(d).value )  
            output_list.append( Field_gnip_profilelocations_address_countrycode(d).value )  
            output_list.append( Field_gnip_profilelocations_address_locality(d).value )  
            output_list.append( Field_gnip_profilelocations_geo_type(d).value )  
            output_list.append( Field_gnip_profilelocations_geo_coordinates(d).value )  

        # user
        if self.options_user:
            output_list.append( Field_actor_displayname(d).value )  
            output_list.append( Field_actor_preferredusername(d).value )  
            output_list.append( Field_actor_id(d).value )  
            
        # user connections, klout
        if self.options_influence:
            output_list.append( Field_gnip_klout_score(d).value )  
            output_list.append( Field_actor_followerscount(d).value )  
            output_list.append( Field_actor_friendscount(d).value )  
            output_list.append( Field_actor_listedcount(d).value )  
            output_list.append( Field_actor_statusesCount(d).value )  
             
        # structure
        if self.options_struct:
            output_list.append( Field_activity_type(d).value )  

        # done building output list 
        return output_list 




#!/usr/bin/env python
import sys
import re
import time
import datetime
import calendar

"""
From twitter code on github
 workerIdBits = 5L
 datacenterIdBits = 5L
 sequenceBits = 12L
 
 maxWorkerId = -1L ^ (-1L << workerIdBits)
 maxDatacenterId = -1L ^ (-1L << datacenterIdBits)

 sequenceMask = -1L ^ (-1L << sequenceBits)
 
 workerIdShift = sequenceBits
 datacenterIdShift = sequenceBits + workerIdBits
 timestampLeftShift = sequenceBits + workerIdBits + datacenterIdBits

 ((timestamp - twepoch) << timestampLeftShift) |
          (datacenterId << datacenterIdShift) |
              (workerId << workerIdShift) | 
               sequence
"""

FMT = "%Y-%m-%dT%H:%M:%S"
TWEPOCH = 1288834974657.

SF_SEQ_BITS = 12
SF_WORK_BITS = 5
SF_DC_BITS = 5
SF_TIME_BITS = 41
SF_BITS = 64

nRE = re.compile("[0-9]{18}")

class Snowflake(object):
    """Snowflake id object provides access to multiple forms of of the bit fields
    present in a snowflake id. Ids can be of the form of a string with leading and
    trailing characters or symbols. The first 18 consecutive digits will be used
    as the id. It is also permitted to provide an int::

        tag:search.twitter.com,2005:113733024721539072
        113733024721539072
        113733024721539072|This is the tweet for which you are looking|en

    where the latter represents a pre-snowflake twitter id. In this case all 
    fields will be null.
    
    When the input does not match a valid snowflake id, the input field is available
    as the id and all other fields are set to None.

    Available fields::

        id 
        sequence 
        worker 
        data_center 
        ts 
        sample_set 
        timestamp 
        timeStruct 
        timeString 
        year 
        month 
        day 
        hour 
        min 
        sec 
        dow 
        doy 

    """

    def __init__(self, id):
        """Create a new snowflake object from string or number represenataion
        of id."""
        ns = nRE.findall(str(id))
        if len(ns) > 0:
            # only process first matching id in string
            self.id = int(ns[0])
            self.sequence = int(self.masked_id(SF_SEQ_BITS, 0))
            self.worker = int(self.masked_id(SF_WORK_BITS, SF_SEQ_BITS))
            self.data_center = int(self.masked_id(SF_DC_BITS, SF_WORK_BITS + SF_SEQ_BITS))
            self.ts = int(self.masked_id(SF_TIME_BITS, SF_DC_BITS + SF_WORK_BITS + SF_SEQ_BITS))
            self.sample_set = self.ts % 100
            # originally  ((self.id >> 22) + TWEPOCH)/1000.0 
            self.timestamp = (self.ts + TWEPOCH)/1000.
            self.timeStruct = time.gmtime(self.timestamp)
            self.timeString = time.strftime(FMT, self.timeStruct)
            self.year = self.timeStruct.tm_year
            self.month = self.timeStruct.tm_mon
            self.day = self.timeStruct.tm_mday
            self.hour = self.timeStruct.tm_hour
            self.min = self.timeStruct.tm_min
            self.sec = self.timeStruct.tm_sec
            self.dow = self.timeStruct.tm_wday
            self.doy = self.timeStruct.tm_yday
            #self.trials = [self.ndigits(self.id, 2)
            #        , self.ndigits(self.ts, 2)
            #        , self.ndigits(self.timestamp, 2)]
        if len(ns) == 0 or self.year < 2010 or self.year > datetime.datetime.now().year + 1:
            # no valid snowflake id found
            self.id = id  # pass through input
            self.sequence = self.worker = self.data_center = self.ts = self.timestamp = None
            self.timeStruct = self.timeString = self.year = self.month = self.day = None
            self.hour = self.min = self.sec = self.dow = self.doy = None

    #def ndigits(self, x, n):
        #return int(x - (10**n) * int(x/(10**n)))

    def masked_id(self, bits, pos):
        # returns an int
        mask = int('1'*bits, 2) << pos
        res = (mask & self.id) >> pos
        #print '%s' % bin(self.id).rjust(65)
        #print '%s' % bin(mask).rjust(65)
        #print '%s' % bin(res).rjust(65)
        return res
    
    def get_id_datetime(self):
        return [self.id, self.timeString]

    def __repr__(self):
        res = "#"*15 + "\n"
        res += "id:      %s\n"%self.id
        res += "seq:     %s\n"%self.sequence
        res += "worker:  %s\n"%self.worker
        res += "DS:      %s\n"%self.data_center
        res += "Seconds: %s\n"%self.timestamp
        res += "time:    %s\n"%self.timeString
        return res

if __name__ == "__main__":
    import csv
    wrtr = csv.writer(sys.stdout)
    for r in sys.stdin:
        ns = nRE.findall(r)
        try:
            for x in ns:
                sf = Snowflake(x)
                wrtr.writerow([sf.id
                    , sf.sequence
                    , sf.worker
                    , sf.data_center
                    , sf.timeString
                    , sf.hour
                    , sf.min
                    , sf.sec])
        except IndexError:
            sys.stderr.write( "ERROR %s"%ns)




#/!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Scott Hendrickson"
__license__="Simplified BSD"
import sys
from .acscsv import AcsCSV

class WPacsCSV(AcsCSV):
    """Word press activites"""
    def __init__(self, delim, options_keypath, options_user, options_rules, options_lang, options_struct):
        super(WPacsCSV, self).__init__(delim,options_keypath)
        self.options_user = options_user
        self.options_rules = options_rules
        self.options_lang = options_lang
        self.options_struct = options_struct

    def procRecordToList(self, d):
        """Creates the list of output fields."""
        record = []
        try:
            if "verb" in d:
                verb = d["verb"]
            else:
                msg = "Unidentified meta message"
                for mtype in ["error", "warning", "info"]:
                    if mtype in d:
                        if "message" in d[mtype]:
                            msg = d[mtype]["message"]
                        elif "message" in d:
                            msg = d["message"]
                        continue
                    mtype = "Unidentified"
                record.append(acscsv.gnipRemove)
                record.append(mtype)
                record.append(msg)
                return record
            if verb == "delete":
                record.append(acscsv.gnipRemove)
                record.append(verb)
                record.append(d["object"]["id"])
                return record
            #
            record.append(d["id"])
            record.append(d["postedTime"])
            obj = d["object"]
            if "content" in obj:
                record.append(obj["content"])
            else:
                record.append("None")
            if "summary" in obj:
                record.append(obj["summary"])
            else:
                record.append("None")
            #
            gnip = d["gnip"]
            actor = d["actor"]
            if self.options_lang:
                glang = "None"
                if "language" in gnip:
                    glang = gnip["language"]["value"]
                record.append(glang)
            if self.options_rules:
                rules = '[]'
                if "matching_rules" in gnip:
                    rules = self.buildListString([ "%s [%s]"%(l["value"], l["tag"]) for l in gnip["matching_rules"]])
                record.append(rules)
            if self.options_struct:
                target = d["target"]
                # put these things in try blocks / ifs 
                # site
                record.append(str(target["wpBlogId"]))
                # blog link
                record.append(target["link"].encode('ascii', 'replace'))
                # object
                record.append(str(obj["wpPostId"]))
                # link to post
                record.append(obj["link"].encode('ascii', 'replace'))
            if self.options_user:
                tmp = "None"
                if (actor is not None) & ("id" in actor):
                    if actor['id'] is not None:
                        tmp = self.splitId(actor["id"])
                record.append(tmp)
            #
            return record
        except KeyError:
            sys.stderr.write("Field missing from record (%d), skipping\n"%self.cnt)
            record.append(acscsv.gnipError)
            return record




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Scott Hendrickson, Josh Montague, Fiona Pigott"
__license__="Simplified BSD"

import sys
import unittest
from StringIO import StringIO
from stocktwits_native import *


# valid native stocktwits record (from sample data dir), used in a bunch of tests
NICE_STRING = """{"id":12802603,"body":"&quot;Hard knocks have a place and value, but hard thinking goes farther in less time.&quot; - Henry Ford","created_at":"2013-04-01T00:32:06Z","user":{"id":76647,"username":"TradingPlays","name":"Paul Elliott","avatar_url":"http://avatars.stocktwits.net/production/76647/thumb-1348961241.png","avatar_url_ssl":"https://s3.amazonaws.com/st-avatars/production/76647/thumb-1348961241.png","identity":"User","classification":[],"join_date":"2011-05-07","followers":273,"following":52,"ideas":3002,"following_stocks":0,"location":"USA","bio":"Proprietary stock trader, specializing in active intraday trading strategies.","website_url":"http://www.tradingplays.com","trading_strategy":{"assets_frequently_traded":["Equities"],"approach":"Momentum","holding_period":"Day Trader","experience":"Professional"}},"source":{"id":1,"title":"StockTwits","url":"http://stocktwits.com"},"entities":{"sentiment":null}}"""


class TestStocktwitsNative(unittest.TestCase):
    """Unit tests of common CSV utility functions"""

    def setUp(self):
        """
        Stocktwits-native constructor takes five args - create one example of each for 
        use in all tests (loop over the self.objs list).
        """ 

        self.delim = '|'

        # use a dict to make it easier to refer to the particular cases
        self.objs = { 
                    "base": StocktwitsNative(self.delim, None, False, False, False)
                    , "user": StocktwitsNative(self.delim, None, True, False, False) 
                    , "struct": StocktwitsNative(self.delim, None, False, True, False) 
                    , "influence": StocktwitsNative(self.delim, None, False, False, True) 
                    , "all": StocktwitsNative(self.delim, None, True, True, True) 
                    , "keypath": StocktwitsNative(self.delim, "source:title", False, False, False) 
                    }

        ## set some vars once, update when needed
        self.base_length = 3
        # count of extra fields added by each of these options
        self.keypath_length = 1
        self.user_length = 3 
        self.struct_length = 3 
        self.influence_length = 3 
        self.all_length = self.base_length \
                            + self.user_length \
                            + self.struct_length \
                            + self.influence_length
        

    def tearDown(self):
        """Nothing to do here."""
        pass


    def test_sample_data(self):
        """
        Check that we can use each test object's procRecord method on each record in the 
        stocktwits_native_sample.json example file without raising an Exception.
        """
        # get a temporary object
        tmp = self.objs["base"]

        # grab the correct data file 
        datafile = "./data/stocktwits_native_sample.json"

        # loop over all test stocktwits processing objects
        for o in self.objs.values():
            # loop over records in test file 
            for i, record in o.file_reader(datafile):
                # if there's a problem parsing, this method will raise an Exception
                record_string = o.procRecord(record)


    def test_reader(self):
        """Test that our file_reader method is working as expected."""
        # grab the base instance 
        o = self.objs["base"]
        
        # without eg a for loop, we use the generator's next() method 
        g = o.file_reader( json_string=NICE_STRING )
        self.assertIsInstance( g.next(), tuple ) 

   
    #
    # test the output (length) for each kind of StocktwitsNative object 
    #
    
    def test_base_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the base instance 
        o = self.objs["base"]

        # use sample record in this module 
        for i, record in o.file_reader( json_string=NICE_STRING ):
            # procRecord returns a delimited string
            record_string = o.procRecord(record)
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length 
                            )  


    def test_user_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["user"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=NICE_STRING )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=NICE_STRING ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.user_length 
                            ) 


    def test_struct_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["struct"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=NICE_STRING )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=NICE_STRING ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.struct_length 
                            )  


    def test_influence_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["influence"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=NICE_STRING )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=NICE_STRING ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.influence_length 
                            )  

    
    def test_all_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["all"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=NICE_STRING )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=NICE_STRING ):
            loop = True
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.all_length 
                            )  


    def test_keypath_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["keypath"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=NICE_STRING )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=NICE_STRING ):
            record_string = o.procRecord(record)
            # should have 1 extra field now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.keypath_length 
                            )  

    #
    # test that all of the fields we need to extract from the records actually work
    #

    def test_struct_fields(self):
        """
        Test for the presence of non-"None" values in a good record when using the 'struct' 
        option, and "None"s in an intentionally-damaged record.
        """ 

        # this activity (chosen from data/) has all the struct fields in it  
        GOOD_STRUCT_STRING = """{"id":12802606,"body":"@Pieman @nnsts just curious.  How much did you lose on such yelling of &#39;out for good?&#39;   Did you set a stop loss?","created_at":"2013-04-01T00:33:39Z","user":{"id":216592,"username":"nnsts","name":"nntrader","avatar_url":"http://avatars.stocktwits.net/production/216592/thumb-1361592350.png","avatar_url_ssl":"https://s3.amazonaws.com/st-avatars/production/216592/thumb-1361592350.png","identity":"User","classification":[],"join_date":"2013-02-22","followers":9,"following":4,"ideas":843,"following_stocks":2,"location":"","bio":null,"website_url":null,"trading_strategy":{"assets_frequently_traded":["Equities","Options","Futures","Bonds"],"approach":"Technical","holding_period":"Swing Trader","experience":"Novice"}},"source":{"id":1,"title":"StockTwits","url":"http://stocktwits.com"},"conversation":{"parent_message_id":12802589,"in_reply_to_message_id":12802589,"parent":false,"replies":3},"entities":{"sentiment":null}}"""

        # break the "conversation" key in this activity, so that options_struct fields should all be "None" 
        BAD_STRUCT_STRING = """{"id":12802606,"body":"@Pieman @nnsts just curious.  How much did you lose on such yelling of &#39;out for good?&#39;   Did you set a stop loss?","created_at":"2013-04-01T00:33:39Z","user":{"id":216592,"username":"nnsts","name":"nntrader","avatar_url":"http://avatars.stocktwits.net/production/216592/thumb-1361592350.png","avatar_url_ssl":"https://s3.amazonaws.com/st-avatars/production/216592/thumb-1361592350.png","identity":"User","classification":[],"join_date":"2013-02-22","followers":9,"following":4,"ideas":843,"following_stocks":2,"location":"","bio":null,"website_url":null,"trading_strategy":{"assets_frequently_traded":["Equities","Options","Futures","Bonds"],"approach":"Technical","holding_period":"Swing Trader","experience":"Novice"}},"source":{"id":1,"title":"StockTwits","url":"http://stocktwits.com"},"GNIP_conversation":{"parent_message_id":12802589,"in_reply_to_message_id":12802589,"parent":false,"replies":3},"entities":{"sentiment":null}}"""

        o = self.objs["struct"]

        # this should have data 
        for i, record in o.file_reader( json_string=GOOD_STRUCT_STRING ):
            record_string = o.procRecord(record)
            [ self.assertNotEqual( "None", x ) 
                for x in record_string.split(self.delim)[-self.struct_length:] 
            ] 
        
        # this should not have data (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_STRUCT_STRING ):
            record_string = o.procRecord(record)
            [ self.assertEqual( "None", x ) 
                for x in record_string.split(self.delim)[-self.struct_length:] 
            ] 


    def test_user_fields(self):
        """
        Test for the presence of non-"None" values in a good record when using the 'user' 
        option, and "None"s in an intentionally-damaged record.
        """ 

        # this activity (chosen from data/) has the user fields in it  
        tmp_url = "http://www.example.com"
        # note: must do the string substition with %s because .format() will look for keys by {}
        GOOD_USER_STRING = """{"id":12802606,"body":"@Pieman @nnsts just curious.  How much did you lose on such yelling of &#39;out for good?&#39;   Did you set a stop loss?","created_at":"2013-04-01T00:33:39Z","user":{"id":216592,"username":"nnsts","name":"nntrader","avatar_url":"http://avatars.stocktwits.net/production/216592/thumb-1361592350.png","avatar_url_ssl":"https://s3.amazonaws.com/st-avatars/production/216592/thumb-1361592350.png","identity":"User","classification":[],"join_date":"2013-02-22","followers":9,"following":4,"ideas":843,"following_stocks":2,"location":"","bio":null,"website_url":"%s","trading_strategy":{"assets_frequently_traded":["Equities","Options","Futures","Bonds"],"approach":"Technical","holding_period":"Swing Trader","experience":"Novice"}},"source":{"id":1,"title":"StockTwits","url":"http://stocktwits.com"},"conversation":{"parent_message_id":12802589,"in_reply_to_message_id":12802589,"parent":false,"replies":3},"entities":{"sentiment":null}}"""%(tmp_url)

        # looks like these fields should always exist, but the 'website_url' could be null. force that & check for "None"
        BAD_USER_STRING = """{"id":12802606,"body":"@Pieman @nnsts just curious.  How much did you lose on such yelling of &#39;out for good?&#39;   Did you set a stop loss?","created_at":"2013-04-01T00:33:39Z","user":{"id":216592,"username":"nnsts","name":"nntrader","avatar_url":"http://avatars.stocktwits.net/production/216592/thumb-1361592350.png","avatar_url_ssl":"https://s3.amazonaws.com/st-avatars/production/216592/thumb-1361592350.png","identity":"User","classification":[],"join_date":"2013-02-22","followers":9,"following":4,"ideas":843,"following_stocks":2,"location":"","bio":null,"website_url":null,"trading_strategy":{"assets_frequently_traded":["Equities","Options","Futures","Bonds"],"approach":"Technical","holding_period":"Swing Trader","experience":"Novice"}},"source":{"id":1,"title":"StockTwits","url":"http://stocktwits.com"},"conversation":{"parent_message_id":12802589,"in_reply_to_message_id":12802589,"parent":false,"replies":3},"entities":{"sentiment":null}}"""

        o = self.objs["user"]

        # this should have data 
        for i, record in o.file_reader( json_string=GOOD_USER_STRING ):
            record_string = o.procRecord(record)
            self.assertNotEqual( "None", record_string.split(self.delim)[-1] )
            self.assertEqual( tmp_url , record_string.split(self.delim)[-1] )
        
        # this should not have data (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_USER_STRING ):
            record_string = o.procRecord(record)
            # payload (json) null ==> Python None ==> check in module
            self.assertEqual( "None", record_string.split(self.delim)[-1] )

   
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Josh Montague"
__license__="MIT License"

import sys
import inspect
import unittest
import json
from StringIO import StringIO
from twitter_acs import *
import  acscsv 


# valid activity from source (eg from data/) 
## 17-241
VALID_ACTIVITY = """
{
   "body": "Brazil Vs chile #WorldCup and @GhettoRadio895 in the background. Can't miss #GNL  @djruffkenya @estherkagamba http://t.co/cT3lgNGwOi", 
   "retweetCount": 0, 
   "generator": {
      "link": "http://twitter.com/download/iphone", 
      "displayName": "Twitter for iPhone"
   }, 
   "twitter_filter_level": "medium", 
   "geo": {
      "type": "Point", 
      "coordinates": [
         37.36813165, 
         -79.62764394
      ]
   }, 
   "favoritesCount": 0, 
   "object": {
      "postedTime": "2014-06-28T18:00:04.000Z", 
      "summary": "Brazil Vs chile #WorldCup and @GhettoRadio895 in the background. Can't miss #GNL  @djruffkenya @estherkagamba http://t.co/cT3lgNGwOi", 
      "link": "http://twitter.com/mokitwou_sigei/statuses/482946548071161857", 
      "id": "object:search.twitter.com,2005:482946548071161857", 
      "objectType": "note"
   }, 
   "actor": {
      "preferredUsername": "mokitwou_sigei", 
      "displayName": "Sam Sigei", 
      "links": [
         {
            "href": null, 
            "rel": "me"
         }
      ], 
      "twitterTimeZone": null, 
      "image": "https://pbs.twimg.com/profile_images/432197756175003648/M_0WMna5_normal.jpeg", 
      "verified": false, 
      "statusesCount": 100, 
      "summary": null, 
      "languages": [
         "en"
      ], 
      "utcOffset": null, 
      "link": "http://www.twitter.com/mokitwou_sigei", 
      "followersCount": 28, 
      "favoritesCount": 22, 
      "friendsCount": 174, 
      "listedCount": 0, 
      "postedTime": "2011-05-21T04:26:03.000Z", 
      "id": "id:twitter.com:302411346", 
      "objectType": "person"
   }, 
   "twitter_lang": "en", 
   "twitter_entities": {
      "symbols": [], 
      "user_mentions": [
         {
            "id": 36316250, 
            "indices": [
               30, 
               45
            ], 
            "id_str": "36316250", 
            "screen_name": "GhettoRadio895", 
            "name": "Ghetto Radio"
         }, 
         {
            "id": 339020526, 
            "indices": [
               82, 
               94
            ], 
            "id_str": "339020526", 
            "screen_name": "djruffkenya", 
            "name": "DJ RUFF di CAPTAIN"
         }, 
         {
            "id": 100987248, 
            "indices": [
               95, 
               109
            ], 
            "id_str": "100987248", 
            "screen_name": "estherkagamba", 
            "name": "Esther Kagamba "
         }
      ], 
      "hashtags": [
         {
            "indices": [
               16, 
               25
            ], 
            "text": "WorldCup"
         }, 
         {
            "indices": [
               76, 
               80
            ], 
            "text": "GNL"
         }
      ], 
      "urls": [], 
      "media": [
         {
            "expanded_url": "http://twitter.com/mokitwou_sigei/status/482946548071161857/photo/1", 
            "display_url": "pic.twitter.com/cT3lgNGwOi", 
            "url": "http://t.co/cT3lgNGwOi", 
            "media_url_https": "https://pbs.twimg.com/media/BrPFU3ECcAAFpi6.jpg", 
            "id_str": "482946547227717632", 
            "sizes": {
               "small": {
                  "h": 453, 
                  "resize": "fit", 
                  "w": 340
               }, 
               "large": {
                  "h": 1024, 
                  "resize": "fit", 
                  "w": 768
               }, 
               "medium": {
                  "h": 800, 
                  "resize": "fit", 
                  "w": 600
               }, 
               "thumb": {
                  "h": 150, 
                  "resize": "crop", 
                  "w": 150
               }
            }, 
            "indices": [
               110, 
               132
            ], 
            "type": "photo", 
            "id": 482946547227717632, 
            "media_url": "http://pbs.twimg.com/media/BrPFU3ECcAAFpi6.jpg"
         }
      ]
   }, 
   "verb": "post", 
   "link": "http://twitter.com/mokitwou_sigei/statuses/482946548071161857", 
   "location": {
      "displayName": "Virginia, USA", 
      "name": "Virginia", 
      "link": "https://api.twitter.com/1.1/geo/id/5635c19c2b5078d1.json", 
      "twitter_country_code": "US", 
      "country_code": "United States", 
      "geo": {
         "type": "Polygon", 
         "coordinates": [
            [
               [
                  -83.67529, 
                  36.540739
               ], 
               [
                  -83.67529, 
                  39.466012
               ], 
               [
                  -75.16644, 
                  39.466012
               ], 
               [
                  -75.16644, 
                  36.540739
               ]
            ]
         ]
      }, 
      "objectType": "place"
   }, 
   "provider": {
      "link": "http://www.twitter.com", 
      "displayName": "Twitter", 
      "objectType": "service"
   }, 
   "postedTime": "2014-06-28T18:00:04.000Z", 
   "id": "tag:search.twitter.com,2005:482946548071161857", 
   "gnip": {
      "klout_profile": {
         "link": "http://klout.com/user/id/16607034464501326", 
         "topics": [
            {
               "link": "http://klout.com/topic/id/9219221220892054727", 
               "displayName": "Dwyane Wade", 
               "klout_topic_id": "9219221220892054727"
            }, 
            {
               "link": "http://klout.com/topic/id/8119426466902417284", 
               "displayName": "Politics", 
               "klout_topic_id": "8119426466902417284"
            }
         ], 
         "klout_user_id": "16607034464501326"
      }, 
      "klout_score": 13, 
      "matching_rules": [
         {
            "tag": null, 
            "value": "(Chile) has:geo"
         }, 
         {
            "tag": null, 
            "value": "(Brazil ) has:geo"
         }
      ], 
      "language": {
         "value": "en"
      }, 
      "urls": [
         {
            "url": "http://t.co/cT3lgNGwOi", 
            "expanded_status": 200, 
            "expanded_url": "http://twitter.com/mokitwou_sigei/status/482946548071161857/photo/1"
         }
      ]
   }, 
   "objectType": "activity"
}
"""
### 17-241


class TestTwitter_acs(unittest.TestCase):
    """Unit tests of common CSV utility functions"""
    def setUp(self):
        """
        """ 
        self.delim = '|'
        valid_activity = json.loads(VALID_ACTIVITY)
        self.objs = {}
        # use a dict to make it easier to refer to the particular object cases
        for name, obj in inspect.getmembers(sys.modules[__name__]):
            if name.startswith("Field_"):
                self.objs[name] = obj(valid_activity)
        # we need to preserve order so the tests match the answers
        self.objs = self.objs.items()
        # get some processing objects
        # delim
        #    , options.keypath
        #    , options.geo
        #    , options.user
        #    , options.rules
        #    , options.urls
        #    , options.lang
        #    , options.influence
        #    , options.struct
 
        args = ["|" 
                , None
                , False
                , False
                , False
                , False
                , False
                , False
                , False
            ]
        self.record_lengths = [
                3
                , 19
                , 22
                , 23
                , 26
                , 29
                , 34
                , 35        
            ]
        self.processing_objs = [TwacsCSV(*args)]
        for i in range(7):
            args[2+i] = True
            #print >>sys.stderr, args
            self.processing_objs.append(TwacsCSV(*args))

    def tearDown(self):
        """
        """
        pass
    #
    # helpful to group test methods that are related into sections
    #
    def test_sample_data(self):
        """
        Check that we can use each test object's procRecord method on each record in the 
        twitter_sample.json example file without raising an Exception.
        """
        # grab the correct data file 
        # TODO: replace hard-coded path to file -- requires running test from acscsv/ dir
        datafile = "../data/twitter_sample.json"
        datafile = "./data/twitter_sample.json"

        # loop over all test twitter processing objects
        for j, o in enumerate(self.processing_objs):
            # loop over records in test file 
            for i, record in o.file_reader(datafile):
                # if there's a problem parsing, this method will raise an Exception
                record_string = o.procRecord(record)
                self.assertEquals(len(record_string.split("|")), self.record_lengths[j])
 
    def test_valid_objects(self):
        for n,x in self.objs:
            self.assertTrue(isinstance(x,acscsv._Field))
    
    def test_number_of_fields(self):
        self.assertEquals(len(self.objs), 70)

    def _test_field_values_(self):
        for n,x in self.objs:
            print "## path = ",x.path
            if len(x.path) == 0 :
                continue
            res = {x.path[-1]: None}
            for y in sorted(x.path, reverse=True)[1:]:
                res = {y:res}
            # test for values
            print "res.append(\"\"\"{}\"\"\")".format(x.value)
            # test for none
            #print "test_doc =",res
            #print "test_obj =",n,"(test_doc)"
            #print "self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')"

    def test_field_values(self):
        res = []
        ## 290-428
        ## path =  ['provider', 'displayName']
        res.append("""Twitter""")
        ## path =  ['actor', 'friendsCount']
        res.append(174)
        ## path =  ['gnip', 'profileLocations']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['gnip', 'matching_rules']
        res.append([{u'tag': None, u'value': u'(Chile) has:geo'}, {u'tag': None, u'value': u'(Brazil ) has:geo'}])
        ## path =  ['actor', 'listedCount']
        res.append(0)
        ## path =  ['generator', 'link']
        res.append("""http://twitter.com/download/iphone""")
        ## path =  ['favoritesCount']
        res.append(0)
        ## path =  ['location', 'twitter_country_code']
        res.append("""US""")
        ## path =  ['location', 'name']
        res.append("""Virginia""")
        ## path =  ['twitter_entities', 'media']
        res.append([{u'expanded_url': u'http://twitter.com/mokitwou_sigei/status/482946548071161857/photo/1', u'sizes': {u'small': {u'h': 453, u'w': 340, u'resize': u'fit'}, u'large': {u'h': 1024, u'w': 768, u'resize': u'fit'}, u'medium': {u'h': 800, u'w': 600, u'resize': u'fit'}, u'thumb': {u'h': 150, u'w': 150, u'resize': u'crop'}}, u'url': u'http://t.co/cT3lgNGwOi', u'media_url_https': u'https://pbs.twimg.com/media/BrPFU3ECcAAFpi6.jpg', u'id_str': u'482946547227717632', u'indices': [110, 132], u'media_url': u'http://pbs.twimg.com/media/BrPFU3ECcAAFpi6.jpg', u'type': u'photo', u'id': 482946547227717632, u'display_url': u'pic.twitter.com/cT3lgNGwOi'}])
        ## path =  ['actor', 'statusesCount']
        res.append(100)
        ## path =  ['gnip', 'profileLocations']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['twitter_lang']
        res.append("""en""")
        ## path =  ['actor', 'displayName']
        res.append("""Sam Sigei""")
        ## path =  ['body']
        res.append("""Brazil Vs chile #WorldCup and @GhettoRadio895 in the background. Can't miss #GNL  @djruffkenya @estherkagamba http://t.co/cT3lgNGwOi""")
        ## path =  ['actor', 'languages']
        res.append("""en""")
        ## path =  ['actor', 'id']
        res.append("""302411346""")
        ## path =  ['actor', 'utcOffset']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['location', 'displayName']
        res.append("""Virginia, USA""")
        ## path =  ['actor', 'link']
        res.append("""http://www.twitter.com/mokitwou_sigei""")
        ## path =  ['twitter_entities', 'urls']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['actor', 'postedTime']
        res.append("""2011-05-21T04:26:03.000Z""")
        ## path =  ['actor', 'preferredUsername']
        res.append("""mokitwou_sigei""")
        ## path =  ['twitter_entities', 'user_mentions']
        res.append([{u'indices': [30, 45], u'screen_name': u'GhettoRadio895', u'id': 36316250, u'name': u'Ghetto Radio', u'id_str': u'36316250'}, {u'indices': [82, 94], u'screen_name': u'djruffkenya', u'id': 339020526, u'name': u'DJ RUFF di CAPTAIN', u'id_str': u'339020526'}, {u'indices': [95, 109], u'screen_name': u'estherkagamba', u'id': 100987248, u'name': u'Esther Kagamba ', u'id_str': u'100987248'}])
        ## path =  ['provider', 'link']
        res.append("""http://www.twitter.com""")
        ## path =  ['gnip', 'profileLocations']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['location', 'link']
        res.append("""https://api.twitter.com/1.1/geo/id/5635c19c2b5078d1.json""")
        ## path =  ['geo', 'coordinates']
        res.append([37.36813165, -79.62764394])
        ## path =  []
        res.append("Tweet (482946548071161857)")
        ## path =  ['actor', 'objectType']
        res.append("""person""")
        ## path =  ['actor', 'favoritesCount']
        res.append(22)
        ## path =  ['objectType']
        res.append("""activity""")
        ## path =  ['id']
        res.append("""482946548071161857_2014-06-28T18:00:04_73""")
        ## path =  ['link']
        res.append("""http://twitter.com/mokitwou_sigei/statuses/482946548071161857""")
        ## path =  ['gnip', 'profileLocations']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['verb']
        res.append("""post""")
        ## path =  ['gnip', 'profileLocations']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['twitter_entities', 'symbols']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['object']
        res.append({u'link': u'http://twitter.com/mokitwou_sigei/statuses/482946548071161857', u'objectType': u'note', u'postedTime': u'2014-06-28T18:00:04.000Z', u'id': u'object:search.twitter.com,2005:482946548071161857', u'summary': u"Brazil Vs chile #WorldCup and @GhettoRadio895 in the background. Can't miss #GNL  @djruffkenya @estherkagamba http://t.co/cT3lgNGwOi"})
        ## path =  ['actor', 'summary']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['actor', 'followersCount']
        res.append(28)
        ## path =  ['gnip', 'profileLocations']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['twitter_entities', 'hashtags']
        res.append([{u'indices': [16, 25], u'text': u'WorldCup'}, {u'indices': [76, 80], u'text': u'GNL'}])
        ## path =  ['postedTime']
        res.append("""2014-06-28T18:00:04.000Z""")
        ## path =  ['twitter_filter_level']
        res.append("""medium""")
        ## path =  ['retweetCount']
        res.append(0)
        ## path =  ['generator', 'displayName']
        res.append("""Twitter for iPhone""")
        ## path =  ['gnip', 'profileLocations']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['actor', 'verified']
        res.append(False)
        ## path =  ['provider', 'objectType']
        res.append("""service""")
        ## path =  ['actor', 'links']
        res.append([{u'href': None, u'rel': u'me'}])
        ## path =  ['location', 'geo', 'coordinates']
        res.append([[-83.67529, 36.540739], [-83.67529, 39.466012], [-75.16644, 39.466012], [-75.16644, 36.540739]])
        ## path =  ['geo', 'type']
        res.append("""Point""")
        ## path =  ['location', 'geo', 'type']
        res.append("""Polygon""")
        ## path =  ['gnip', 'klout_profile', 'link']
        res.append("""http://klout.com/user/id/16607034464501326""")
        ## path =  ['actor', 'twitterTimeZone']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['gnip', 'klout_profile', 'klout_user_id']
        res.append("""16607034464501326""")
        ## path =  ['location', 'country_code']
        res.append("""United States""")
        ## path =  ['actor', 'image']
        res.append("""https://pbs.twimg.com/profile_images/432197756175003648/M_0WMna5_normal.jpeg""")
        ## path =  ['id']
        res.append("""482946548071161857""")
        ## path =  ['actor', 'location', 'objectType']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['gnip', 'klout_profile', 'topics']
        res.append([{u'displayName': u'Dwyane Wade', u'link': u'http://klout.com/topic/id/9219221220892054727', u'klout_topic_id': u'9219221220892054727'}, {u'displayName': u'Politics', u'link': u'http://klout.com/topic/id/8119426466902417284', u'klout_topic_id': u'8119426466902417284'}])
        ## path =  ['inReplyTo', 'link']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['gnip', 'profileLocations']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['gnip', 'language', 'value']
        res.append("""en""")
        ## path =  ['gnip', 'klout_score']
        res.append(13)
        ## path =  ['gnip', 'profileLocations']
        res.append("""GNIPEMPTYFIELD""")
        ## path =  ['gnip', 'urls']
        res.append([{u'url': u'http://t.co/cT3lgNGwOi', u'expanded_status': 200, u'expanded_url': u'http://twitter.com/mokitwou_sigei/status/482946548071161857/photo/1'}])
        ## path =  ['location', 'objectType']
        res.append("""place""")
        ## path =  ['actor', 'location', 'displayName']
        res.append("""GNIPEMPTYFIELD""")
        ## 289-428
        for x,t in zip(self.objs, res):
            #print t
            self.assertEquals(x[1].value, t)
 
    def test_none_value(self):
        ## 433-706 
        ## path =  ['provider', 'displayName']
        test_doc = {'displayName': {'displayName': None}}
        test_obj = Field_provider_displayname (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'friendsCount']
        test_doc = {'actor': {'friendsCount': None}}
        test_obj = Field_actor_friendscount (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'profileLocations']
        test_doc = {'gnip': {'profileLocations': None}}
        test_obj = Field_gnip_profilelocations_displayname (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'matching_rules']
        test_doc = {'gnip': {'matching_rules': None}}
        test_obj = Field_gnip_rules (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'listedCount']
        test_doc = {'actor': {'listedCount': None}}
        test_obj = Field_actor_listedcount (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['generator', 'link']
        test_doc = {'generator': {'link': None}}
        test_obj = Field_generator_link (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['favoritesCount']
        test_doc = {'favoritesCount': None}
        test_obj = Field_favoritescount (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['location', 'twitter_country_code']
        test_doc = {'location': {'twitter_country_code': None}}
        test_obj = Field_location_twitter_country_code (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['location', 'name']
        test_doc = {'location': {'name': None}}
        test_obj = Field_location_name (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['twitter_entities', 'media']
        test_doc = {'media': {'media': None}}
        test_obj = Field_twitter_entities_media (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'statusesCount']
        test_doc = {'actor': {'statusesCount': None}}
        test_obj = Field_actor_statusesCount (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'profileLocations']
        test_doc = {'gnip': {'profileLocations': None}}
        test_obj = Field_gnip_profilelocations_address_region (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['twitter_lang']
        test_doc = {'twitter_lang': None}
        test_obj = Field_twitter_lang (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'displayName']
        test_doc = {'actor': {'displayName': None}}
        test_obj = Field_actor_displayname (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['body']
        test_doc = {'body': None}
        test_obj = Field_body (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'languages']
        test_doc = {'actor': {'languages': None}}
        test_obj = Field_actor_language (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'id']
        test_doc = {'actor': {'id': None}}
        test_obj = Field_actor_id (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'utcOffset']
        test_doc = {'actor': {'utcOffset': None}}
        test_obj = Field_actor_utcoffset (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['location', 'displayName']
        test_doc = {'displayName': {'displayName': None}}
        test_obj = Field_location_displayname (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'link']
        test_doc = {'actor': {'link': None}}
        test_obj = Field_actor_link (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['twitter_entities', 'urls']
        test_doc = {'twitter_entities': {'urls': None}}
        test_obj = Field_twitter_entities_urls (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'postedTime']
        test_doc = {'actor': {'postedTime': None}}
        test_obj = Field_actor_postedtime (test_doc)
        self.assertEquals(test_obj.value, 'INVALID_DATE_FORMAT')
        ## path =  ['actor', 'preferredUsername']
        test_doc = {'actor': {'preferredUsername': None}}
        test_obj = Field_actor_preferredusername (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['twitter_entities', 'user_mentions']
        test_doc = {'twitter_entities': {'user_mentions': None}}
        test_obj = Field_twitter_entities_user_mentions (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['provider', 'link']
        test_doc = {'link': {'link': None}}
        test_obj = Field_provider_link (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'profileLocations']
        test_doc = {'gnip': {'profileLocations': None}}
        test_obj = Field_gnip_profilelocations_address_countrycode (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['location', 'link']
        test_doc = {'link': {'link': None}}
        test_obj = Field_location_link (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['geo', 'coordinates']
        test_doc = {'coordinates': {'coordinates': None}}
        test_obj = Field_geo_coordinates (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  []
        ## path =  ['actor', 'objectType']
        test_doc = {'actor': {'objectType': None}}
        test_obj = Field_actor_objecttype (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'favoritesCount']
        test_doc = {'actor': {'favoritesCount': None}}
        test_obj = Field_actor_favoritesCount (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['objectType']
        test_doc = {'objectType': None}
        test_obj = Field_objecttype (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['link']
        test_doc = {'link': None}
        test_obj = Field_link (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'profileLocations']
        test_doc = {'gnip': {'profileLocations': None}}
        test_obj = Field_gnip_profilelocations_geo_coordinates (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['verb']
        test_doc = {'verb': None}
        test_obj = Field_verb (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'profileLocations']
        test_doc = {'gnip': {'profileLocations': None}}
        test_obj = Field_gnip_profilelocations_objecttype (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['twitter_entities', 'symbols']
        test_doc = {'symbols': {'symbols': None}}
        test_obj = Field_twitter_entities_symbols (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['object']
        test_doc = {'object': None}
        test_obj = Field_object (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'summary']
        test_doc = {'actor': {'summary': None}}
        test_obj = Field_actor_summary (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'followersCount']
        test_doc = {'actor': {'followersCount': None}}
        test_obj = Field_actor_followerscount (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'profileLocations']
        test_doc = {'gnip': {'profileLocations': None}}
        test_obj = Field_gnip_profilelocations_address_subregion (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['twitter_entities', 'hashtags']
        test_doc = {'hashtags': {'hashtags': None}}
        test_obj = Field_twitter_entities_hashtags (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['postedTime']
        test_doc = {'postedTime': None}
        test_obj = Field_postedtime (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['twitter_filter_level']
        test_doc = {'twitter_filter_level': None}
        test_obj = Field_twitter_filter_level (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['retweetCount']
        test_doc = {'retweetCount': None}
        test_obj = Field_retweetcount (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['generator', 'displayName']
        test_doc = {'displayName': {'displayName': None}}
        test_obj = Field_generator_displayname (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'profileLocations']
        test_doc = {'gnip': {'profileLocations': None}}
        test_obj = Field_gnip_profilelocations_geo_type (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'verified']
        test_doc = {'actor': {'verified': None}}
        test_obj = Field_actor_verified (test_doc)
        self.assertEquals(test_obj.value, False)
        ## path =  ['provider', 'objectType']
        test_doc = {'objectType': {'objectType': None}}
        test_obj = Field_provider_objecttype (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'links']
        test_doc = {'actor': {'links': None}}
        test_obj = Field_actor_links (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['location', 'geo', 'coordinates']
        test_doc = {'coordinates': {'geo': {'coordinates': None}}}
        test_obj = Field_location_geo_coordinates (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['geo', 'type']
        test_doc = {'geo': {'type': None}}
        test_obj = Field_geo_type (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['location', 'geo', 'type']
        test_doc = {'geo': {'location': {'type': None}}}
        test_obj = Field_location_geo_type (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'klout_profile', 'link']
        test_doc = {'gnip': {'klout_profile': {'link': None}}}
        test_obj = Field_gnip_klout_profile_link (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'twitterTimeZone']
        test_doc = {'actor': {'twitterTimeZone': None}}
        test_obj = Field_actor_twittertimezone (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'klout_profile', 'klout_user_id']
        test_doc = {'gnip': {'klout_profile': {'klout_user_id': None}}}
        test_obj = Field_gnip_klout_profile_klout_user_id (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['location', 'country_code']
        test_doc = {'country_code': {'country_code': None}}
        test_obj = Field_location_country_code (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'image']
        test_doc = {'actor': {'image': None}}
        test_obj = Field_actor_image (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['id']
        test_doc = {'id': None}
        test_obj = Field_id (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'location', 'objectType']
        test_doc = {'actor': {'location': {'objectType': None}}}
        test_obj = Field_actor_location_objecttype (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'klout_profile', 'topics']
        test_doc = {'gnip': {'klout_profile': {'topics': None}}}
        test_obj = Field_gnip_klout_profile_topics (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['inReplyTo', 'link']
        test_doc = {'inReplyTo': {'link': None}}
        test_obj = Field_inreplyto_link (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'profileLocations']
        test_doc = {'gnip': {'profileLocations': None}}
        test_obj = Field_gnip_profilelocations_address_country (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'language', 'value']
        test_doc = {'gnip': {'language': {'value': None}}}
        test_obj = Field_gnip_language_value (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'klout_score']
        test_doc = {'gnip': {'klout_score': None}}
        test_obj = Field_gnip_klout_score (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'profileLocations']
        test_doc = {'gnip': {'profileLocations': None}}
        test_obj = Field_gnip_profilelocations_address_locality (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['gnip', 'urls']
        test_doc = {'gnip': {'urls': None}}
        test_obj = Field_gnip_urls (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['location', 'objectType']
        test_doc = {'location': {'objectType': None}}
        test_obj = Field_location_objecttype (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## path =  ['actor', 'location', 'displayName']
        test_doc = {'actor': {'displayName': {'displayName': None}}}
        test_obj = Field_actor_location_displayname (test_doc)
        self.assertEquals(test_obj.value, 'GNIPEMPTYFIELD')
        ## 433-706

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Scott Hendrickson"
__license__="Simplified BSD"
import sys
from .acscsv import AcsCSV
class DiacsCSV(AcsCSV):
    """
    DiacsCSV determins the parse order and included fields when parsing Disqus Activity Streams
    JSON records to CSV fields.
    """
    def __init__(self, delim, options_keypath, options_user, options_rules, options_lang, options_struct, options_status):
        super(DiacsCSV, self).__init__(delim,options_keypath)
        self.options_user = options_user
        self.options_rules = options_rules
        self.options_lang = options_lang
        self.options_struct = options_struct
        self.options_status = options_status

    def procRecordToList(self, d):
        """Take a JSON Activity Streams payload as a Python dictionary. Check 
        activity for system information and compliance handling. If necessary, 
        return the system info or compliance message. Otherwise, if the activity 
        is valid, return list of fields as specified by the input flags.
        
        Flags::

            delim
            options_keypath
            options_user
            options_rules
            options_lang
            options_struct
            options_status

        """
        record = []
        try:
            if "verb" in d:
                verb = d["verb"]
            else:
                msg = "Unidentified meta message"
                for mtype in ["error", "warning", "info"]:
                    if mtype in d:
                        if "message" in d[mtype]:
                            msg = d[mtype]["message"]
                        elif "message" in d:
                            msg = d["message"]
                        continue
                    mtype = "Unidentified"
                record.append(acscsv.gnipRemove)
                record.append(mtype)
                record.append(msg)
                return record
            if verb == "delete":
                record.append(acscsv.gnipRemove)
                record.append(verb)
                #record.append(self.cleanField(d["object"]["id"]))
                record.append(d["object"]["id"])
                return record
            #
            record.append(d["id"])
            record.append(d["postedTime"])
            #record.append(self.cleanField(d["body"]))
            record.append(d["body"])
            #
            gnip = d["gnip"]
            actor = d["actor"]
            if self.options_lang:
                glang = "None"
                if "language" in gnip:
                    glang = gnip["language"]["value"]
                record.append(glang)
            if self.options_rules:
                rules = '[]'
                if "matching_rules" in gnip:
                    rules = self.buildListString([ "%s [%s]"%(l["value"], l["tag"]) for l in gnip["matching_rules"]])
                record.append(rules)
            if self.options_struct:
                target = d["target"]
                # site
                tmp = self.splitId(target["website"]["id"])
                record.append(tmp)
                # thread
                tmp2 = self.splitId(target["id"])
                record.append(tmp2)
                # thread link
                if "link" in target and target["link"] is not None:
                    record.append(target["link"])
                else:
                    record.append("None")
                # in reply to
                if "inReplyTo" in d:
                    in_reply_to = d["inReplyTo"]
                    # comment
                    tmp2 = self.splitId(in_reply_to["id"])
                    record.append(tmp2)
                    # reply to user
                    tmp3 = self.splitId(in_reply_to["author"]["id"])
                    if tmp3 == "-1":
                        tmp3 = "Anon"
                    record.append(tmp3)
                else:
                    record.append("None")
                    record.append("None")
            if self.options_user:
                tmp = self.splitId(actor["id"])
                if tmp == "-1":
                    tmp = "Anon"
                record.append(tmp)
            if self.options_status:
                record.append(verb)
                if "disqusType" in d:
                    record.append(str(d["disqusType"]))
                else:
                    record.append("None")
                if "disqusTypePrev" in d:
                    record.append(str(d["disqusTypePrev"]))
                else:
                    record.append("None")
            #
            return record
        except KeyError:
            sys.stderr.write("Field missing from record (%d), skipping\n"%self.cnt)
            record.append(acscsv.gnipError)
            return record




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Josh Montague"
__license__="MIT License"

import sys
import unittest
from StringIO import StringIO
from reflect_json import *


# valid activity from source (eg from data/) 
VALID_ACTIVITY ={}


class Testreflect(unittest.TestCase):
    """Unit tests of common CSV utility functions"""

    def setUp(self):
        """
        """ 

        self.delim = '|'

        # use a dict to make it easier to refer to the particular object cases
        self.objs = { 
                    "base": "foo" 
                    , "user": "baz" 
                    }

        # set any instance attributes here, avoid hard-coding in test methods 
        self.bar = None        

    def tearDown(self):
        """
        """
        pass

    
    #
    # helpful to group test methods that are related into sections
    #
    
    def test_method1(self):
        self.assertFalse(False)

        
    def test_method2(self):
        self.assertTrue(True)


 
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Josh Montague"
__license__="MIT License"

import sys
import unittest
from StringIO import StringIO
from foursquare_acs import *


# valid activity from source (eg from data/) 
VALID_ACTIVITY = """{"id":"tag:gnip.foursquare.com:2013:checkin/53910e5c498eb98e8cadf5cc","postedTime":"2014-06-06T00:42:04+00:00","verb":"checkin","actor":{"objectType":"person","gender":"male"},"object":{"id":"tag:gnip.foursquare.com:2013:venue/4d88cbd33acc6dcb3b7d0f1c","displayName":"Planet Fitness","objectType":"place","geo":{"type":"Point","coordinates":[-84.21241229795072,33.71550258242101]},"address":{"locality":"Decatur","region":"GA","postalCode":"30035","country":"United States"},"foursquareCategories":[{"id":"tag:gnip.foursquare.com:2013:category/4bf58dd8d48988d176941735","displayName":"Gym","image":"https://ss1.4sqi.net/img/categories_v2/building/gym_88.png"}]},"provider":{"link":"https://foursquare.com","displayName":"Foursquare","objectType":"service"},"foursquareCheckinUtcOffset":-14400,"gnip":{}}"""


class TestFoursquareACS(unittest.TestCase):
    """Unit tests of common CSV utility functions"""

    def setUp(self):
        """
        Foursquare constructor takes seven args - create one example of each for 
        use in all tests (loop over the self.objs list).
        """ 

        self.delim = '|'

        # use a dict to make it easier to refer to the particular object cases
        self.objs = { 
                    "base": FsqacsCSV(self.delim, None, False, False, False)
                    , "geo": FsqacsCSV(self.delim, None, True, False, False)
                    , "user": FsqacsCSV(self.delim, None, False, True, False)
                    , "rules": FsqacsCSV(self.delim, None, False, False, True)
                    , "all": FsqacsCSV(self.delim, None, True, True, True)
                    , "keypath": FsqacsCSV(self.delim, "actor:gender", False, False, False) 
                    }

        # set any instance attributes here, avoid hard-coding in test methods 
        self.base_length = 6
        # count of extra fields added by each of these options
        self.keypath_length = 1
        self.geo_length = 4
        self.user_length = 1
        self.rules_length = 1
        self.all_length = self.base_length \
                            + self.geo_length \
                            + self.user_length \
                            + self.rules_length 

    def tearDown(self):
        """Nothing to do here."""
        pass

    
    def test_sample_data(self):
        """
        Check that we can use each test object's procRecord method on each record in the 
        foursquare_sample.json example file without raising an Exception.
        """
        # get a temporary object
        tmp = self.objs["base"]

        # grab the correct data file 
        datafile = "./data/foursquare_sample.json" 

        # loop over all test foursquare processing objects
        for o in self.objs.values():
            # loop over records in test file 
            for i, record in o.file_reader(datafile):
                # if there's a problem parsing, this method will raise an Exception
                record_string = o.procRecord(record)


    def test_reader(self):
        """Test that our file_reader method is working as expected."""
        # grab the base instance 
        o = self.objs["base"]
        
        # without eg a for loop, we use the generator's next() method 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

   
    #
    # test the output (length) for each kind of FsqacsCSV object 
    #
    
    def test_base_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the base instance 
        o = self.objs["base"]

        # use sample record in this module 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            # procRecord returns a delimited string
            record_string = o.procRecord(record)
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length 
                            )  


    def test_user_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["user"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.user_length 
                            ) 


    def test_geo_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["geo"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.geo_length 
                            )  

    
    def test_rules_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["rules"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.rules_length 
                            )  

    
    def test_all_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["all"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            loop = True
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.all_length 
                            )  


    def test_keypath_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["keypath"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 1 extra field now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.keypath_length 
                            )  

    def test_category_field(self):
        """
        Test for the presence of non-"None" values in a good record,
        and "None"s in an intentionally-damaged record.
        """ 
        
        # 
        GOOD_CAT_STRING = """{"id":"tag:gnip.foursquare.com:2013:checkin/51a0afd68bfdb8df55921c1d","postedTime":"2013-05-25T12:34:29+00:00","verb":"checkin","actor":{"objectType":"person","gender":"female"},"object":{"id":"tag:gnip.foursquare.com:2013:venue/4d1778691356a093cc22dc82","displayName":"Leman Kültür","objectType":"place","geo":{"type":"Point","coordinates":[32.688869786457,39.87231478030219]},"address":{"locality":"Ankara","region":"Türkiye","postalCode":"06800","country":"Turkey"},"foursquareCategories":[{"id":"tag:gnip.foursquare.com:2013:category/4bf58dd8d48988d155941735","displayName":"Gastropub"},{"id":"tag:gnip.foursquare.com:2013:category/4bf58dd8d48988d16d941735","displayName":"Café"},{"id":"tag:gnip.foursquare.com:2013:category/4bf58dd8d48988d16e941735","displayName":"Fast Food Restaurant"}]},"provider":{"link":"https://foursquare.com","displayName":"Foursquare","objectType":"service"},"foursquareCheckinUtcOffset":10800}"""

        # make fourSquareCategories an empty list
        BAD_CAT_STRING = """{"id":"tag:gnip.foursquare.com:2013:checkin/51a0afd68bfdb8df55921c1d","postedTime":"2013-05-25T12:34:29+00:00","verb":"checkin","actor":{"objectType":"person","gender":"female"},"object":{"id":"tag:gnip.foursquare.com:2013:venue/4d1778691356a093cc22dc82","displayName":"Leman Kültür","objectType":"place","geo":{"type":"Point","coordinates":[32.688869786457,39.87231478030219]},"address":{"locality":"Ankara","region":"Türkiye","postalCode":"06800","country":"Turkey"},"foursquareCategories":[]},"provider":{"link":"https://foursquare.com","displayName":"Foursquare","objectType":"service"},"foursquareCheckinUtcOffset":10800}"""

        o = self.objs["base"]

        # this should have data 
        for i, record in o.file_reader( json_string=GOOD_CAT_STRING ):
            record_string = o.procRecord(record)
            self.assertNotEqual( "None", record_string.split(self.delim)[4] )
            self.assertNotEqual( "['None']", record_string.split(self.delim)[4] )
        
        # this should not have data (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_CAT_STRING ):
            record_string = o.procRecord(record)
            self.assertEqual( "['None']", record_string.split(self.delim)[4] )

    def test_geo_fields(self):
        """
        Test for the presence of non-"None" values in a good record,
        and "None"s in an intentionally-damaged record.
        """ 

        GOOD_GEO_STRING = """{"id":"tag:gnip.foursquare.com:2013:checkin/51a0afd68bfdb8df55921c1d","postedTime":"2013-05-25T12:34:29+00:00","verb":"checkin","actor":{"objectType":"person","gender":"female"},"object":{"id":"tag:gnip.foursquare.com:2013:venue/4d1778691356a093cc22dc82","displayName":"Leman Kültür","objectType":"place","geo":{"type":"Point","coordinates":[32.688869786457,39.87231478030219]},"address":{"locality":"Ankara","region":"Türkiye","postalCode":"06800","country":"Turkey"},"foursquareCategories":[{"id":"tag:gnip.foursquare.com:2013:category/4bf58dd8d48988d155941735","displayName":"Gastropub"},{"id":"tag:gnip.foursquare.com:2013:category/4bf58dd8d48988d16d941735","displayName":"Café"},{"id":"tag:gnip.foursquare.com:2013:category/4bf58dd8d48988d16e941735","displayName":"Fast Food Restaurant"}]},"provider":{"link":"https://foursquare.com","displayName":"Foursquare","objectType":"service"},"foursquareCheckinUtcOffset":10800}"""

        BAD_LOCALITY_STRING = """{"id":"tag:gnip.foursquare.com:2013:checkin/51a0afd68bfdb8df55921c1d","postedTime":"2013-05-25T12:34:29+00:00","verb":"checkin","actor":{"objectType":"person","gender":"female"},"object":{"id":"tag:gnip.foursquare.com:2013:venue/4d1778691356a093cc22dc82","displayName":"Leman Kültür","objectType":"place","geo":{"type":"Point","coordinates":[32.688869786457,39.87231478030219]},"address":{"region":"Türkiye","postalCode":"06800","country":"Turkey"},"foursquareCategories":[]},"provider":{"link":"https://foursquare.com","displayName":"Foursquare","objectType":"service"},"foursquareCheckinUtcOffset":10800}"""

# missing postal code
        BAD_POSTALCODE_STRING = """{"id":"tag:gnip.foursquare.com:2013:checkin/51a0afd68bfdb8df55921c1d","postedTime":"2013-05-25T12:34:29+00:00","verb":"checkin","actor":{"objectType":"person","gender":"female"},"object":{"id":"tag:gnip.foursquare.com:2013:venue/4d1778691356a093cc22dc82","displayName":"Leman Kültür","objectType":"place","geo":{"type":"Point","coordinates":[32.688869786457,39.87231478030219]},"address":{"locality":"Ankara","region":"Türkiye","country":"Turkey"},"foursquareCategories":[]},"provider":{"link":"https://foursquare.com","displayName":"Foursquare","objectType":"service"},"foursquareCheckinUtcOffset":10800}"""

# missing country key
        BAD_COUNTRY_STRING = """{"id":"tag:gnip.foursquare.com:2013:checkin/51a0afd68bfdb8df55921c1d","postedTime":"2013-05-25T12:34:29+00:00","verb":"checkin","actor":{"objectType":"person","gender":"female"},"object":{"id":"tag:gnip.foursquare.com:2013:venue/4d1778691356a093cc22dc82","displayName":"Leman Kültür","objectType":"place","geo":{"type":"Point","coordinates":[32.688869786457,39.87231478030219]},"address":{"locality":"Ankara","region":"Türkiye","postalCode":"06800"},"foursquareCategories":[]},"provider":{"link":"https://foursquare.com","displayName":"Foursquare","objectType":"service"},"foursquareCheckinUtcOffset":10800}"""

# region key missing
        BAD_REGION_STRING = """{"id":"tag:gnip.foursquare.com:2013:checkin/51a0afd68bfdb8df55921c1d","postedTime":"2013-05-25T12:34:29+00:00","verb":"checkin","actor":{"objectType":"person","gender":"female"},"object":{"id":"tag:gnip.foursquare.com:2013:venue/4d1778691356a093cc22dc82","displayName":"Leman Kültür","objectType":"place","geo":{"type":"Point","coordinates":[32.688869786457,39.87231478030219]},"address":{"locality":"Ankara","postalCode":"06800","country":"Turkey"},"foursquareCategories":[]},"provider":{"link":"https://foursquare.com","displayName":"Foursquare","objectType":"service"},"foursquareCheckinUtcOffset":10800}"""

        o = self.objs["geo"]

        # this should have data 
        for i, record in o.file_reader( json_string=GOOD_GEO_STRING ):
            record_string = o.procRecord(record)
            self.assertNotEqual( "None", record_string.split(self.delim)[6] )
        
        # this should not have data (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_LOCALITY_STRING ):
            record_string = o.procRecord(record)
            self.assertEqual( "None", record_string.split(self.delim)[6] )

        # this should not have data (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_POSTALCODE_STRING ):
            record_string = o.procRecord(record)
            self.assertEqual( "None", record_string.split(self.delim)[9] )
        
        # this should not have data (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_REGION_STRING ):
            record_string = o.procRecord(record)
            self.assertEqual( "None", record_string.split(self.delim)[7] )
        
        # this should not have data (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_COUNTRY_STRING ):
            record_string = o.procRecord(record)
            self.assertEqual( "None", record_string.split(self.delim)[8] )


 
if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Scott Hendrickson, Josh Montague"
__license__="Simplified BSD"

import pkg_resources
try:
    __version__ = pkg_resources.require("gnacs")[0].version
except pkg_resources.DistributionNotFound:
    __version__ = "N/A"
import sys
import codecs
import fileinput
import re
import os
import argparse
from acscsv import twitter_acs
# needed only for the pretty-printing
import json as json_printer
# use fastest option available for parsing
try:
    import ujson as json
except ImportError:
    try:
        import json
    except ImportError:
        import simplejson as json

def gnacs_args():
    """Parse comand line arguemnts for defining input and output of command line utility."""
    parser = argparse.ArgumentParser(
            description="Parse seqeunce of JSON formated activities.")
    parser.add_argument("file_name", metavar= "file_name", nargs="?"
            , default=[] 
            , help="Input file name (optional).")
    parser.add_argument("-a","--status", action="store_true", dest="status"
            , default=False
            , help="Version, status, etc.")
    parser.add_argument("-g","--geo", action="store_true", dest="geo"
            , default=False
			, help="Include geo fields")
    parser.add_argument("-i", "--influence", action="store_true", dest="influence"
            , default=False
			, help="Show user's influence metrics (Twitter only)")
    parser.add_argument("-c","--csv", action="store_true", dest="csv"
            , default=False
			, help="Comma-delimited output (, default is | without quotes)")
    parser.add_argument("-l","--lang", action="store_true", dest="lang"
            , default=False
			, help="Include language fields")
    parser.add_argument("-j","--geojson", action="store_true", dest="geojson"
            , default=False
			, help="Output is geojson format (Foursquare and Twitter only) \
                    Caution: dataset must fit in memory.")
    parser.add_argument("-o","--origin", action="store_true", dest="origin"
            , default=False
			, help="Include source/origin fields")
    parser.add_argument("-p","--pretty", action="store_true", dest="pretty"
            , default=False
			, help="Pretty JSON output of full records")
    parser.add_argument("-s", "--urls", action="store_true", dest="urls"
            , default=False
			, help="Include urls fields")
    parser.add_argument("-t","--structure", action="store_true", dest="struct"
            , default=False
			, help="Include thread linking fields")
    parser.add_argument("-r","--rules", action="store_true", dest="rules"
            , default=False
			, help="Include rules fields")
    parser.add_argument("-u","--user", action="store_true", dest="user"
            , default=False
			, help="Include user fields")
    parser.add_argument("-v","--version", action="store_true", dest="ver"
            , default=False
			, help="Show version number")
    parser.add_argument("-x","--explain", action="store_true", dest="explain", 
            default=False
			, help="Show field names in output for sample input records")
    parser.add_argument("-z","--publisher", dest="pub"
            , default="twitter"
			, help="Publisher (default is twitter), twitter, newsgator, disqus, \
                    wordpress, wpcomments, tumblr, foursquare, getglue, stocktwits, stocktwits-native")
    parser.add_argument("-k","--keypath", dest="keypath"
            , default=None
			, help="returns a value from a path of the form 'key:value'")
    return parser

if __name__ == "__main__":
    """Use gnacs delimited-field parsing libraries as a command line tool to parse a series of JSON
    formatted actvities from file, compressed file or standard input (stdin)."""
    
    options = gnacs_args().parse_args()
    if options.ver:
        print("*"*70)
        print("Gnacs Version: %s"%__version__)
        print("Please see https://github.com/DrSkippy27/Gnacs for updates or")
        print("sudo pip install gnacs --upgrade to install the latest version.")
        print("*"*70)
        sys.exit()
    #
    delim = "|"     # default delimiter
    if options.csv:
        delim = "," # csv delimiter
    elif options.geojson:
        options.geo = True 
        # note: geojson option creates an in-memory structure
        sys.stdout.write('{"type": "FeatureCollection", "features": [')
    #
    if options.pub.lower().startswith("word") or options.pub.lower().startswith("wp"):
        processing_obj = wordpress_acs.WPacsCSV(delim
			, options.keypath
			, options.user
			, options.rules
			, options.lang
			, options.struct
            )
    elif options.pub.lower().startswith("disq"):
        processing_obj = disqus_acs.DiacsCSV(delim
			, options.keypath
			, options.user
			, options.rules
			, options.lang
			, options.struct
			, options.status
            )
    else:
        processing_obj = twitter_acs.TwacsCSV(delim
			, options.keypath
			, options.geo
			, options.user
			, options.rules
			, options.urls
			, options.lang
			, options.influence
			, options.struct
            )
    #
    first_geo = True 
    for line_number, record in processing_obj.file_reader(options.file_name): 
        if options.pretty:
            print(json_printer.dumps(record, indent=3, ensure_ascii=False) )
            continue 
        try:
            if options.explain:
                #### TODO: fix -x option for new extractors ####
                sys.stderr.write("\n****\n\n'explain' functionality currently unavailable\n\n****\n")
                sys.exit()
                ################################################
                record = reflect_json.reflect_json(record)
                sys.stdout.write("%s\n"%processing_obj.procRecord(record))
            elif options.geojson:
                # geo-tag coords
                geo_rec = processing_obj.asGeoJSON(record)
                if geo_rec is not None:
                    if not first_geo: 
                        sys.stdout.write(",")
                    sys.stdout.write(json.dumps(geo_rec))
                    first_geo = False
            else:
                # ensure formatter is working on a unicode object 
                sys.stdout.write(u"{}\n".format(processing_obj.procRecord(record, emptyField="None")))
        # handle I/O exceptions associated with writing to stdout (e.g. when output is piped to 'head')
        # TODO: handle this via contextmanager (within AcsCSV)? 
        except IOError as e:
            try:
                sys.stdout.close()
            except IOError:
                pass
            try:
                sys.stderr.close()
            except IOError:
                pass
            break
    # close the geojson data structure
    if options.geojson:
        sys.stdout.write(']}\n')            




# -*- coding: UTF-8 -*-
__author__="Josh Montague, Scott Hendrickson, Nikhil Sulegaon"
__license__="Simplified BSD"

import sys
from .acscsv import _Field
from .snowflake import *
from datetime import datetime
import re


########################################
#   activity type 
########################################

class Field_activity_type(_Field):
    """
    Take a dict, assign to self.value the appropriate value of Tweet, Retweet, or Reply. This 
    class is being held onto for now, but may not be needed in future.
    """
    label = 'Activity Type'
    path = []
    
    def __init__(self, json_record):
        super(
            Field_activity_type
            , self).__init__(json_record)
        # self.value is None
        if json_record is not None:
            verb = Field_verb(json_record).value 
            rec_id = Field_id(json_record).value  
            inReplyTo = "None"
            obj_objtype = "None"
            if "inReplyTo" in json_record:
                # get the url
                inReplyTo = json_record["inReplyTo"]["link"]
                # get the original id from the url
                rec_id = inReplyTo.split("/")[-1]
            if "object" in json_record:
                obj = json_record["object"]
                if "objectType" in obj:
                    obj_objtype = obj["objectType"]
            # now we can determine self.value
            if verb == "share" and obj_objtype == "activity":
                self.value = "Retweet"
            elif inReplyTo == "None":
                self.value = "Tweet"
            else:
                self.value = "Reply"
            # tack on the upstream activity id (or this id, for Tweets)  
            self.value += " ({})".format(rec_id)



########################################
#   top-level fields 
########################################

class Field_verb(_Field):
    """Take a dict, assign to self.value the value in the top-level verb key.""" 
    label = 'Verb'
    path = ['verb']
    # overwrite this default value because records missing this field should be called out (badness) 
    # 2014-05-29, leave this out for new usage (JM)
    #default_value = "Unidentified meta message"
    

class Field_id(_Field):
    """Take a dict, assign to self.value the value in the top-level id key.""" 
    label = 'Tweet ID'
    path = ['id_str']
    
    def __init__(self, json_record):
        super(
            Field_id
            , self).__init__(json_record)
        tmp = self.value.split(":")
        if len(tmp) >= 3:
            self.value = tmp[2]

class Field_snowflake(Field_id):
    def __init__(self, json_record):
        super(
            Field_snowflake
            , self).__init__(json_record)
        sf = Snowflake(self.value)
        self.value = "_".join([str(sf.id), sf.timeString, str(sf.sample_set)])


class Field_objecttype(_Field):
    """Take a dict, assign to self.value the value in the top-level objectType key.""" 
    label = 'objectType' 
    path = ['objectType']
    

class Field_object(_Field):
    """Take a dict, assign to self.value the value in the top-level object key.""" 
    label = 'Object'
    path = ['object']


class Field_postedtime(_Field):
    """
    Take a dict, assign to self.value the value in the top-level postedTime key. Timestamp is 
    formatted according to input_fmt, which is set in the constructor.
    """
    label = 'Posted Time'
    path = ['created_at']

    # keep this around for possible regex+uniform output extension
#    def __init__(self, json_record):
#        super(
#            Field_postedtime 
#            , self).__init__(json_record)
#        # self.value is a datetime string 
#        input_fmt = "%Y-%m-%dT%H:%M:%S.000Z"
#        # add a regex match on the input to have consistent output across all input streams? 
#        self.value = datetime.strptime( 
#                        self.value, input_fmt 
#                        ).strftime( 
#                            self.default_t_fmt ) 


class Field_body(_Field):
    """Take a dict, assign to self.value the value in top-level body key."""
    label = 'Body'
    path = ['body']
    

class Field_link(_Field):
    """Take a dict, assign to self.value the value in top-level link key."""
    label = 'Activity URL'
    path = ['link']
    

class Field_twitter_lang(_Field):
    """Take a dict, assign to self.value the value of top-level twitter_lang key."""
    label = 'Twitter-Inferred Activity Language'
    path = ['lang']
    

class Field_favoritescount(_Field):
    """Take a dict, assign to self.value the value of top-level favoritesCount key."""
    label = 'Favorite Count'
    path = ['favoritesCount']


class Field_retweetcount(_Field):
    """Take a dict, assign to self.value the value of top-level retweetCount key."""
    label = 'Retweet Count'
    path = ['retweetCount']


class Field_twitter_filter_level(_Field):
    """Take a dict, assign to self.value the value of top-level twitter_filter_level key."""
    label = 'Twitter Filter Level'
    path = ['twitter_filter_level']


class Field_inreplyto_link(_Field):
    """
    Take a dict, assign to self.value the value of inReplyTo.link. 
    Should only appear in Replies.
    """
    label = 'In-Reply-To URL'
    path = ['inReplyTo', 'link']
    


########################################
#   'provider' fields 
########################################

class Field_provider_objecttype(_Field):
    """Take a dict, assign to self.value the value of provider.objectType key."""
    label = 'Provider Object Type'
    path = ['provider', 'objectType']


class Field_provider_displayname(_Field):
    """Take a dict, assign to self.value the value of provider.displayName key."""
    label = 'Provider Name'
    path = ['provider', 'displayName']


class Field_provider_link(_Field):
    """Take a dict, assign to self.value the value of provider.link key."""
    label = 'Provider Link'
    path = ['provider', 'link']



########################################
#   'generator' fields 
########################################

class Field_generator_displayname(_Field):
    """Take a dict, assign to self.value the value of generator.displayName key."""
    label = 'Generator Name'
    path = ['generator', 'displayName']
    

class Field_generator_link(_Field):
    """Take a dict, assign to self.value the value of generator.link key."""
    label = 'Generator Link'
    path = ['generator', 'link']



########################################
#   'gnip' fields 
########################################

class Field_gnip_rules(_Field):
    """Take a dict, assign to self.value the list of dicts at gnip.matching_rules"""
    label = 'Matching PT Rules'
    path = ['gnip', 'matching_rules']
    

class Field_gnip_urls(_Field):
    """Take a dict, assign to self.value the list of dicts at gnip.urls"""
    label = 'List of URLs (Gnip)'
    path = ['gnip', 'urls']

# TODO: extractors that use Field_gnip_urls to get both the shortened & unwound URLs
    

class Field_gnip_language_value(_Field):
    """Take a dict, assign to self.value the value of gnip.language.value"""
    label = 'Gnip-Inferred Activity Language'
    path = ['gnip', 'language', 'value']


# klout
class Field_gnip_klout_score(_Field):
    """Take a dict, assign to self.value the value of gnip.klout_score"""
    label = 'User Klout Score'
    path = ['gnip', 'klout_score']


class Field_gnip_klout_profile_topics(_Field):
    """Take a dict, assign to self.value the list at gnip.klout_profile.topics"""
    label = 'User Klout Topics'
    path = ['gnip', 'klout_profile', 'topics']
    

class Field_gnip_klout_profile_klout_user_id(_Field):
    """Take a dict, assign to self.value the value of gnip.klout_profile.klout_user_id"""
    label = 'User Klout ID'
    path = ['gnip', 'klout_profile', 'klout_user_id']
    

class Field_gnip_klout_profile_link(_Field):
    """Take a dict, assign to self.value the value of gnip.klout_profile.link"""
    label = 'User Klout Profile Link'
    path = ['gnip', 'klout_profile', 'link']


# profileLocations
class _Field_gnip_profilelocations_base(_Field):
    """
    Take a dict, assign to self.value the dict at gnip.profileLocations[0]. This class 
    serves as the base for subfields of profileLocations.
    """ 
    label = 'Profile Geo: Data Structure'
    path = ['gnip', 'profileLocations']

    def __init__(self, json_record):
        super(
            _Field_gnip_profilelocations_base 
            , self).__init__(json_record)
        if self.value != self.default_value:
            # if we found the list, we really want the first (only) thing in it
            self.value = self.value[0]


class Field_gnip_profilelocations_displayname(_Field_gnip_profilelocations_base):
    """Take a dict, assign to self.value the value of gnip.profileLocations[0].displayName."""
    label = 'Profile Geo: Name'

    def __init__(self, json_record):
        super(
            Field_gnip_profilelocations_displayname 
            , self).__init__(json_record)
        if self.value != self.default_value:
            self.value = self.walk_path( self.value, path = ['displayName'] ) 


class Field_gnip_profilelocations_objecttype(_Field_gnip_profilelocations_base):
    """Take a dict, assign to self.value the value of gnip.profileLocations[0].objectType"""
    label = 'Profile Geo: Object Type'

    def __init__(self, json_record):
        super(
            Field_gnip_profilelocations_objecttype
            , self).__init__(json_record)
        if self.value != self.default_value:
            self.value = self.walk_path( self.value, path = ['objectType'] ) 


class Field_gnip_profilelocations_geo_type(_Field_gnip_profilelocations_base):
    """Take a dict, assign to self.value the value of gnip.profileLocations[0].geo.type"""
    label = 'Profile Geo: Type'

    def __init__(self, json_record):
        super(
            Field_gnip_profilelocations_geo_type
            , self).__init__(json_record)
        if self.value != self.default_value:
            self.value = self.walk_path( self.value, path = ['geo', 'type'] ) 


class Field_gnip_profilelocations_geo_coordinates(_Field_gnip_profilelocations_base):
    """Take a dict, assign to self.value the value of gnip.profileLocations[0].geo.coordinates"""
    label = 'Profile Geo: Coordinates'

    def __init__(self, json_record):
        super(
            Field_gnip_profilelocations_geo_coordinates
            , self).__init__(json_record)
        if self.value != self.default_value:
            self.value = self.walk_path( self.value, path = ['geo', 'coordinates'] ) 


class Field_gnip_profilelocations_address_country(_Field_gnip_profilelocations_base):
    """Take a dict, assign to self.value the value of gnip.profileLocations[0].address.country"""
    label = 'Profile Geo: Country'

    def __init__(self, json_record):
        super(
            Field_gnip_profilelocations_address_country
            , self).__init__(json_record)
        if self.value != self.default_value:
            self.value = self.walk_path( self.value, path = ['address', 'country'] ) 


class Field_gnip_profilelocations_address_countrycode(_Field_gnip_profilelocations_base):
    """Take a dict, assign to self.value the value of gnip.profileLocations[0].address.countryCode"""
    label = 'Profile Geo: Country Code'

    def __init__(self, json_record):
        super(
            Field_gnip_profilelocations_address_countrycode
            , self).__init__(json_record)
        if self.value != self.default_value:
            self.value = self.walk_path( self.value, path = ['address', 'countryCode'] ) 


class Field_gnip_profilelocations_address_locality(_Field_gnip_profilelocations_base):
    """Take a dict, assign to self.value the value of gnip.profileLocations[0].address.locality"""
    label = 'Profile Geo: Locality'

    def __init__(self, json_record):
        super(
            Field_gnip_profilelocations_address_locality
            , self).__init__(json_record)
        if self.value != self.default_value:
            self.value = self.walk_path( self.value, path = ['address', 'locality'] ) 


class Field_gnip_profilelocations_address_region(_Field_gnip_profilelocations_base):
    """Take a dict, assign to self.value the value of gnip.profileLocations[0].address.region"""
    label = 'Profile Geo: Region'

    def __init__(self, json_record):
        super(
            Field_gnip_profilelocations_address_region
            , self).__init__(json_record)
        if self.value != self.default_value:
            self.value = self.walk_path( self.value, path = ['address', 'region'] ) 


class Field_gnip_profilelocations_address_subregion(_Field_gnip_profilelocations_base):
    """Take a dict, assign to self.value the value of gnip.profileLocations[0].address.subRegion"""
    label = 'Profile Geo: Subregion'

    def __init__(self, json_record):
        super(
            Field_gnip_profilelocations_address_subregion
            , self).__init__(json_record)
        if self.value != self.default_value:
            self.value = self.walk_path( self.value, path = ['address', 'subRegion'] ) 



########################################
#   'actor' fields 
########################################

class Field_actor_id(_Field):
    """
    Take a dict, assign to self.value the numerical value of actor.id (after stripping off 
    the leading 'id:twitter...' characters).
    """
    label = 'User ID'
    path = ['actor', 'id']
    
    def __init__(self, json_record):
        super(
            Field_actor_id
            , self).__init__(json_record)
        if self.value != self.default_value:
            # self.value has an id:twitter....
            self.value = self.value.split(":")[2]


class Field_actor_objecttype(_Field):
    """Take a dict, assign to self.value the value of actor.objecttype """
    label = 'User Object Type'
    path = ['actor', 'objectType']
    

class Field_actor_postedtime(_Field):
    """Take a dict, assign to self.value the value of actor.postedTime """ 
    label = 'User Account Creation Date'
    path = ['actor', 'postedTime']
    
    def __init__(self, json_record):
        super(
            Field_actor_postedtime 
            , self).__init__(json_record)
        # self.value is a string (of a timestamp) 
        input_fmt = "%Y-%m-%dT%H:%M:%S.000Z"
        try:
            # default_t_fmt defined in _Field constructor
            self.value = datetime.strptime(self.value, input_fmt).strftime(self.default_t_fmt) 
        except ValueError:
            self.value = "INVALID_DATE_FORMAT"


class Field_actor_displayname(_Field):
    """Take a dict, assign to self.value the value of actor.displayName """
    label = 'User Display Name'
    path = ['actor', 'displayName']


class Field_actor_preferredusername(_Field):
    """Take a dict, assign to self.value the value of actor.preferredUsername"""
    label = 'User Account Name'
    path = ['actor', 'preferredUsername']
 

class Field_actor_summary(_Field):
    """Take a dict, assign to self.value the value of actor.summary"""
    label = 'User Bio'
    path = ['actor', 'summary']


class Field_actor_link(_Field):
    """Take a dict, assign to self.value the value of actor.link"""
    label = 'User Account URL'
    path = ['actor', 'link']


class Field_actor_image(_Field):
    """Take a dict, assign to self.value the value of actor.image"""
    label = 'User Account Image'
    path = ['actor', 'image']


class Field_actor_language(_Field):
    """
    Take a dict, assign to self.value the first (currently only) value in the list at 
    actor.languages
    """
    label = 'User-chosen Language'
    path = ['actor', 'languages']
    
    def __init__(self, json_record):
        super(
            Field_actor_language
            , self).__init__(json_record)
        # self.value is a list, but have only ever seen it with one value, so take that one. 
        if self.value != self.default_value:
            self.value = self.value[0]


class Field_actor_links(_Field):
    """Take a dict, assign to self.value the list of dicts at actor.links """
    label = 'User Account Bio URLs'
    path = ['actor', 'links']

    def __init__(self, json_record):
        super(
            Field_actor_links
            , self).__init__(json_record)


class Field_actor_twittertimezone(_Field):
    """Take a dict, assign to self.value the value of actor.twitterTimeZone"""
    label = 'User-chosen Timezone'
    path = ['actor', 'twitterTimeZone']


class Field_actor_utcoffset(_Field):
    """Take a dict, assign to self.value the value of actor.utcOffset """
    label = 'User-chosen Offset From UTC'
    path = ['actor', 'utcOffset']
    

class Field_actor_verified(_Field):
    """
    Take a dict, assign to self.value a boolean repr of the value of actor.verified. 
    Default value is False.
    """
    label = 'Actor Verified'
    path = ['actor', 'verified']
    default_value = False


class Field_actor_location_displayname(_Field):
    """Take a dict, assign to self.value the value of actor.location.displayName"""
    label = 'User-chosen Location Name'
    path = ['actor', 'location', 'displayName']


class Field_actor_location_objecttype(_Field):
    """Take a dict, assign to self.value the value of actor.location.objectType"""
    label = 'User-chosen Location Type'
    path = ['actor', 'location', 'objectType']


class Field_actor_followerscount(_Field):
    """Take a dict, assign to self.value the value of actor.followersCount."""
    label = 'User Follower Count'
    path = ['actor', 'followersCount']


class Field_actor_friendscount(_Field):
    """Take a dict, assign to self.value the value of actor.friendsCount."""
    label = 'User Following Count'
    path = ['actor', 'friendsCount']

    
class Field_actor_listedcount(_Field):
    """Take a dict, assign to self.value the value of actor.listedCount."""
    label = 'User Listed Count'
    path = ['actor', 'listedCount']


class Field_actor_statusesCount(_Field):
    """Take a dict, assign to self.value the value of actor.statusesCount"""
    label = 'User Statuses Count'
    path = ['actor', 'statusesCount']


class Field_actor_favoritesCount(_Field):
    """Take a dict, assign to self.value the value of actor.favoritesCount"""
    label = 'User Favorites Count'
    path = ['actor', 'favoritesCount']



########################################
#   'twitter_entities' fields 
########################################

# URLs
class Field_twitter_entities_urls(_Field):
    """Take a dict, assign to self.value the list of dicts at twitter_entities.urls"""
    label = 'List of URLs (Twitter)'
    path = ['twitter_entities', 'urls']


# hashtags
class Field_twitter_entities_hashtags(_Field):
    """Take a dict, assign to self.value the list of dicts at twitter_entities.hashtags"""
    label = 'List of Hashtags (Twitter)'
    path = ['twitter_entities', 'hashtags']


# symbols
class Field_twitter_entities_symbols(_Field):
    """Take a dict, assign to self.value the list of dicts at twitter_entities.symbols"""
    label = 'List of Symbols (Twitter)'
    path = ['twitter_entities', 'symbols']


# mentions
class Field_twitter_entities_user_mentions(_Field):
    """Take a dict, assign to self.value the list of dicts at twitter_entities.user_mentions"""
    label = 'List of Mentions (Twitter)'
    path = ['twitter_entities', 'user_mentions']


# media
class Field_twitter_entities_media(_Field):
    """Take a dict, assign to self.value the list of dicts at twitter_entities.media"""
    label = 'List of Media (Twitter)'
    path = ['twitter_entities', 'media']



########################################
#   'geo' fields 
########################################

class Field_geo_type(_Field):
    """Take a dict, assign to self.value the value of geo.type"""
    label = 'Geo-tag: Type'
    path = ['geo', 'type']
    

class Field_geo_coordinates(_Field):
    """
    Take a dict, assign to self.value the list at geo.coordinates. This is the coordinate pair 
    of the user-enabled tweet geotag.
    """
    label = 'Geo-tag: Coordinates'
    path = ['geo', 'coordinates']
   


########################################
#   'location' fields 
########################################

class Field_location_displayname(_Field):
    """Take a dict, assign to self.value the value of location.displayName"""
    label = 'Geo-tag: Full Name'
    path = ['location', 'displayName']


class Field_location_name(_Field):
    """Take a dict, assign to self.value the value of location.name"""
    label = 'Geo-tag: Short Name'
    path = ['location', 'name']


class Field_location_objecttype(_Field):
    """Take a dict, assign to self.value the value of location.objectType"""
    label = 'Geo-tag: Location Type'
    path = ['location', 'objectType']


class Field_location_twitter_country_code(_Field):
    """Take a dict, assign to self.value the value of location.twitter_country_code."""
    label = 'Geo-tag: Country Code'
    path = ['location', 'twitter_country_code']


class Field_location_country_code(_Field):
    """Take a dict, assign to self.value the value of location.country_code."""
    label = 'Geo-tag: Country'
    path = ['location', 'country_code']


class Field_location_link(_Field):
    """Take a dict, assign to self.value the value of location.link."""
    label = 'Geo-tag: Place URL'
    path = ['location', 'link']


# location.geo
class Field_location_geo_type(_Field):
    """Take a dict, assign to self.value the value of location.geo.type."""
    label = 'Geo-tag: Place Type'
    path = ['location', 'geo', 'type']


class Field_location_geo_coordinates(_Field):
    """
    Take a dict, assign to self.value the list of coord pairs (vertices) in the value 
    of location.geo.coordinates.
    """
    label = 'Geo-tag: Place Coordinates'
    path = ['location', 'geo', 'coordinates']

    def __init__(self, json_record):
        super(
            Field_location_geo_coordinates 
            , self).__init__(json_record)
        # this list has only been observed to contain another list of the bounding vertices 
        if self.value != self.default_value:
            self.value = self.value[0]



########################################
#   'object' fields 
########################################

# these fields appear to be redundant with the fields already defined 




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Scott Hendrickson"
__license__="Simplified BSD"

import sys
import datetime
import fileinput
from io import StringIO
# Experimental: Use numba to speed up some fo the basic function
# that are run many times per record
# from numba import jit
# use fastest option available
try:
    import ujson as json
except ImportError:
    try:
        import json
    except ImportError:
        import simplejson as json

gnipError = "GNIPERROR"
gnipRemove = "GNIPREMOVE"
gnipDateTime = datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.000Z")
INTERNAL_EMPTY_FIELD = "GNIPEMPTYFIELD"

class _Field(object):
    """
    Base class for extracting the desired value at the end of a series of keys in a JSON Activity 
    Streams payload. Set the application-wide default value (for e.g. missing values) here, 
    but also use child classes to override when necessary. Subclasses also need to define the 
    key-path (path) to the desired location by overwriting the path attr.
    """

    # set some default values; these can be overwritten in custom classes 
    # twitter format
    default_t_fmt = "%Y-%m-%dT%H:%M:%S.000Z" 
    default_value = INTERNAL_EMPTY_FIELD
    path = []                       # dict key-path to follow for desired value

    label = 'DummyKeyPathLabel'         # this must match if-statement in constructor

    def __init__(self, json_record):
        if self.label == 'DummyKeyPathLabel':
            self.label = ':'.join(self.path)
        self.value = None                    # str representation of the field, often = str( self.value_list ) 
        if json_record is not None:
            self.value = self.walk_path(json_record)
        else:
            self.value = self.default_value

    def __repr__(self):
        return unicode(self.value)

    def walk_path(self, json_record, path=None):
        res = json_record
        if path is None:
            path = self.path
        for k in path:
            if res is None:
                break
            if k not in res or ( type(res[k]) is list and len(res[k]) == 0 ):
                # parenthetical clause for values with empty lists e.g. twitter_entities
                return self.default_value
            res = res[k]
        # handle the special case where the walk_path found null (JSON) which converts to 
        # a Python None. Only use "None" (str version) if it's assigned to self.default_value 
        res = res if res is not None else self.default_value
        return res
    
    def walk_path_slower(self, json_record, path=None):
        """Slower version fo walk path. Depricated."""
        if path is None:
            path = self.path
        try:
            execstr = "res=json_record" + '["{}"]'*len(path)
            exec(execstr.format(*path))
        except (KeyError, TypeError):
            res = None
        if res is None:
            res = self.default_value
        return res

    def fix_length(self, iterable, limit=None):
        """
        Take an iterable (typically a list) and an optional maximum length (limit). 
        If limit is not given, and the input iterable is not equal to self.default_value
        (typically "None"), the input iterable is returned. If limit is given, the return
        value is a list that is either truncated to the first limit items, or padded 
        with self.default_value until it is of size limit. Note: strings are iterables, 
        so if you pass this function a string, it will (optionally) truncate the 
        number of characters in the string according to limit. 
        """
        res = [] 

        if limit is None:
            # no limits on the length of the result, so just return the original iterable
            res = iterable
        else:
            #if len(iterable) == 0:
            if iterable == self.default_value or len(iterable) == 0:
                # if walk_path() finds the final key, but the value is an empty list 
                #   (common for e.g. the contents of twitter_entities) 
                #   overwrite self.value with a list of self.default_value and of length limit
                res = [ self.default_value ]*limit
            else:
                # found something useful in the iterable, either pad the list or truncate 
                #   to end up with something of the proper length                                                                                                                                   
                current_length = len( iterable ) 
                if current_length < limit:
                    res = iterable + [ self.default_value 
                                        for _ in range(limit - current_length) ]
                else:  
                    res = iterable[:limit]
        return res


class _LimitedField(_Field):
    """  
    Takes JSON record (in python dict form) and optionally a maximum length (limit, 
    with default length=5). Uses parent class _Field() to assign the appropriate value 
    to self.value. When self.value is a list of dictionaries, 
    inheriting from _LimitedField() class allows for the extraction and combination of 
    an arbitrary number of fields within self.value into self.value_list.

    Ex: if your class would lead to having 
    self.value = [ {'a': 1, 'b': 2, 'c': 3}, {'a': 4, 'b': 5, 'c': 6} ], and what you'd like 
    is a list that looks like [ 1, 2, 4, 5 ], inheriting from _LimitedField() allows you 
    to overwrite the fields list ( fields=["a", "b"] ) to obtain this result. 
    Finally, self.value is set to a string representation of the final self.value_list.
    """
    #TODO: is there a better way that this class and the fix_length() method in _Field class
    #       could be combined?
    #TODO: set limit=None by default and just return as many as there are, otherwise (by specifying 
    #    limit), return a maximum of limit.
    # TODO:
    # - consolidate _LimitedField() & fix_length() if possible 

    def __init__(self, json_record, limit=1):
        self.fields = None
        super(
            _LimitedField 
            , self).__init__(json_record)
        # self.value is possibly a list of dicts for each activity media object 
        if self.fields:
            # start with default list full of the default_values
            self.value_list = [ self.default_value ]*( len(self.fields)*limit )
            if self.value != self.default_value: 
                for i,x in enumerate(self.value):   # iterate over the dicts in the list
                    if i < limit:                   # ... up until you reach limit 
                        for j,y in enumerate(self.fields):      # iterate over the dict keys 
                            self.value_list[ len( self.fields )*i + j ] = x[ self.fields[j] ] 
            # finally, str-ify the list
            self.value = str( self.value_list )
    
class AcsCSV(object):
    """Base class for all delimited list objects. Basic delimited list utility functions"""

    def __init__(self, delim, options_keypath):
        self.delim = delim
        if delim == "":
            print >>sys.stderr, "Warning - Output has Null delimiter"
        self.rmchars = "\n\r {}".format(self.delim)
        self.options_keypath = options_keypath
        
    def string_hook(self, record_string, mode_dummy):
        """
        Returns a file-like StringIO object built from the activity record in record_string.
        This is ultimately passed down to the FileInput.readline() method. The mode_dummy 
        parameter is only included so the signature matches other hooks. 
        """
        return StringIO( record_string ) 

    def file_reader(self, options_filename=None, json_string=None):
        """
        Read arbitrary input file(s) or standard Python str. When passing file_reader() a 
        JSON string, assign it to the json_string arg. Yields a tuple of (line number, record).
        """
        line_number = 0
        if json_string is not None: 
            hook = self.string_hook 
            options_filename = json_string 
        else:
            hook = fileinput.hook_compressed
        for r in fileinput.FileInput(options_filename, openhook=hook):  
            line_number += 1
            try:
                recs = [json.loads(r.strip())]
            except ValueError:
                try:
                    # maybe a missing line feed?
                    recs = [json.loads(x) for x in r.strip().replace("}{", "}GNIP_SPLIT{")
                        .split("GNIP_SPLIT")]
                except ValueError:
                    sys.stderr.write("Invalid JSON record (%d) %s, skipping\n"
                        %(line_number, r.strip()))
                    continue
            for record in recs:
                if len(record) == 0:
                    continue
                # hack: let the old source modules still have a self.cnt for error msgs
                self.cnt = line_number
                yield line_number, record

    def cleanField(self,f):
        """Clean fields of new lines and delmiter."""
        res = INTERNAL_EMPTY_FIELD
        try:
            res = f.strip(
                ).replace("\n"," "
                ).replace("\r"," "
                ).replace(self.delim, " "
                )
        except AttributeError:
            try:
                # odd edge case that f is a number
                # then can't call string functions
                float(f)
                res = str(f)
            except TypeError:
                pass
        return res

    def buildListString(self,l):
        """Generic list builder returns a string representation of list"""
        # unicode output of list (without u's)
        res = '['
        for r in l:
            # handle the various types of lists we might see
            try:
                if isinstance(r, list):
                    res += "'" + self.buildListString(r) + "',"
                elif isinstance(r, str) or isinstance(r, unicode):
                    res += "'" + r + "'," 
                else:
                    res += "'" + str(r) + "',"
            except NameError:
                if isinstance(r, list):
                    res += "'" + self.buildListString(r) + "',"
                elif isinstance(r, str):
                    res += "'" + r + "'," 
                else:
                    res += "'" + str(r) + "',"
        if res.endswith(','):
            res = res[:-1]
        res += ']'
        return res

    def splitId(self, x, index=1):
        """Generic functions for splitting id parts"""
        tmp = x.split("/")
        if len(tmp) > index:
            return tmp[index]
        else:
            return x

    def asString(self, l, emptyField):
        """Returns a delimited list object as a properly delimited string."""
        if l is None:
            return None
        for i, x in enumerate(l):
            if x == INTERNAL_EMPTY_FIELD:
                l[i] = emptyField
        return self.delim.join(l)

    def get_source_list(self, x):
        """Wrapper for the core activity parsing function."""
        source_list = self.procRecordToList(x)
        if self.options_keypath:
            source_list.append(self.keyPath(x))
        # ensure no pipes, newlines, etc
        return [ self.cleanField(x) for x in source_list ]

    def procRecord(self, x, emptyField="None"):
        return self.asString(self.get_source_list(x), emptyField)

    def asGeoJSON(self, x):
        """Get results as GeoJSON representation."""
        record_list = self.procRecordToList(x)
        if self.__class__.__name__ == "TwacsCSV" and self.options_geo:
            if self.geoCoordsList is None:
                return
            lon_lat = self.geoCoordsList[::-1]
        elif self.__class__.__name__ == "FsqacsCSV" and self.options_geo:
            lon_lat = self.geo_coords_list
        else:
            return {"Error":"This publisher doesn't have geo"}
        return { 
                "type": "Feature"
                , "geometry": { "type": "Point", "coordinates": lon_lat }
                , "properties": { "id": record_list[0] } 
                }
    
    def keyPath(self,d):
        """Get a generic key path specified at run time. Consider using jq instead?"""
        #key_list = self.options_keypath.split(":")
        delim = ":"
        #print >> sys.stderr, "self.__class__ " + str(self.__class__)
        if self.__class__.__name__ == "NGacsCSV":
            delim = ","
        key_stack = self.options_keypath.split(delim)
        #print >> sys.stderr, "key_stack " + str(key_stack)
        x = d
        while len(key_stack) > 0:
            try:
                k = key_stack.pop(0)
                try:
                    idx = int(k)
                except ValueError:
                    # keys are ascii strings
                    idx = str(k)
                x = x[idx]
            except (IndexError, TypeError, KeyError) as e:
                #sys.stderr.write("Keypath error at %s\n"%k)
                return "PATH_EMPTY"
        return unicode(x)




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
__author__="Brian Lehman"
__license__="MIT License"

import sys
import unittest
from StringIO import StringIO
from disqus_acs import *


# valid an activity from source (eg from data/) turned into a strong -- so use json.loads(str) to create a dict
VALID_ACTIVITY = """{"body": ">\\"incompetent situations like Benghazi.\\"<\\n\\n\\nBlow it out yourass, Goober\\n\\n\\n(I mean no disrespect by that)\\n\\n\\nThe last GOP administration is responsible for 4,000+ dead American service people, 2,800 dead Americans in NYC and 100,000+ dead Iraqi\'s.\\n\\n\\nAnd you\'rre going to get your panties in a bunch over 4 dead Americans killed in a terrorist attack in the ME -  because you think you can make some political hay over it?\\n\\n\\nWith all due respect - phucque you and the horse you rode in on", "disqusTypePrev": "approved", "inReplyTo": {"objectType": "comment", "id": "tag:gnip.disqus.com:2012:comment/1haa42", "disqusId": "1473071832", "author": {"id": "tag:gnip.disqus.com:2012:account/a8m86y", "disqusId": "64154961"}}, "object": {"link": "http://www.politico.com/story/2014/07/benghazi-trey-gowdy-108625.html#comment-1473078627", "id": "tag:gnip.disqus.com:2012:comment/1hae9j", "disqusId": "1473078627", "objectType": "comment"}, "disqusType": "approved", "verb": "update", "disqusMessageTimestamp": "2014-07-07T23:34:26+00:00", "id": "tag:gnip.disqus.com:2012:comment/1hae9j/update/2014-07-07T19:33:59/ab61a0faa8752910cf949d274bfd51d1090b2dedc63bf7fe5d21a608441f6ac3", "target": {"website": {"id": "tag:gnip.disqus.com:2012:forum/itsy", "disqusId": "1422873"}, "link": "http://www.politico.com/story/2014/07/benghazi-trey-gowdy-108625.html", "disqusId": "2825453635", "postedTime": "2014-07-07T21:59:23+00:00", "id": "tag:gnip.disqus.com:2012:thread/a69my0m", "objectType": "article"}, "gnip": {"language": {"value": "en"}}, "actor": {"preferredUsername": "disqus_xs2HXcqTks", "id": "tag:gnip.disqus.com:2012:account/ani2ch", "disqusId": "95458297", "objectType": "person"}, "postedTime": "2014-07-07T23:33:59+00:00"}"""
        
class TestDisqus(unittest.TestCase):
    """Unit tests of common CSV utility functions"""

    def setUp(self):
        """
        DiacsCSV constructor takes  args - create one example of each for 
        use in all tests (loop over the self.objs list).
        """ 

        self.delim = '|'

        # use a dict to make it easier to refer to the particular object cases
        #class DiacsCSV(acscsv.AcsCSV): def __init__(self, delim, options_keypath, options_user, options_rules, options_lang, options_struct, options_status)
        self.objs = { 
                    "base": DiacsCSV(self.delim,None,False,False,False,False,False) 
                    , "keypath": DiacsCSV(self.delim,"verb",False,False,False,False,False) 
                    , "user": DiacsCSV(self.delim,None,True,False,False,False,False) 
                    , "rules": DiacsCSV(self.delim,None,False,True,False,False,False) 
                    , "lang": DiacsCSV(self.delim,None,False,False,True,False,False) 
                    , "struct": DiacsCSV(self.delim,None,False,False,False,True,False) 
                    , "status": DiacsCSV(self.delim,None,False,False,False,False,True) 
                    , "all": DiacsCSV(self.delim,None,True,True,True,True,True) 
                    }

        # set any instance attributes here, avoid hard-coding in test methods 
        self.base_length = 3
        # count of extra fields added by each of these options
        self.keypath_length = 1
        self.user_length = 1
        self.rules_length = 1
        self.lang_length = 1
        self.struct_length = 5
        self.status_length = 3
        self.all_length = self.base_length \
                            + self.user_length \
                            + self.rules_length \
                            + self.lang_length \
                            + self.struct_length \
                            + self.status_length \
                                
    def tearDown(self):
        """
        Nothing to do here.
        """
        pass
    
    #
    # helpful to group test methods that are related into sections
    #

    def test_sample_data(self):
        """
        Check that we can use each test object's procRecord method on each record in the 
        disqus_sample.json example file without raising an Exception.
        """

        # get a temporary object
        tmp = self.objs["base"]

        # grab the correct data file 
        datafile = "./data/disqus_sample.json"

        # loop over all test disqus  processing objects
        for o in self.objs.values():
            # loop over records in test file 
            for i, record in o.file_reader(datafile):
                # if there's a problem parsing, this method will raise an Exception
                record_string = o.procRecord(record)

    def test_reader(self):
        """Test that our file_reader method is working as expected."""
        # grab the base instance 
        o = self.objs["base"]

        # without eg a for loop, we use the generator's next() method 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

   
    #
    # test the output (length) for each kind of DiacsCSV object 
    #

    def test_base_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the base instance 
        o = self.objs["base"]

        # use sample record in this module 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ): 
        # procRecord returns a delimited string
            record_string = o.procRecord(record)
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length 
                            )  

    def test_keypath_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["keypath"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 1 extra field now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.keypath_length 
                            )  

    def test_user_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["user"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.user_length 
                            ) 

    def test_rules_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["rules"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.rules_length 
                            )  

    def test_lang_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["lang"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.base_length + self.lang_length 
                            )  

    def test_struct_fields(self):
        """
        Test for the presence of non-"None" values in a good record when using the 'struct' 
        option, and "None"s in an intentionally-damaged record.
        """ 
        
        # this activity (chosen from data/) has all the struct fields in it  
        GOOD_STRUCT_STRING = VALID_ACTIVITY

        # remove the "inReplyTo" key in this activity, so that the last two options_struct fields should be "None" 
        BAD_STRUCT_STRING = """{"body": ">\\"incompetent situations like Benghazi.\\"<\\n\\n\\nBlow it out yourass, Goober\\n\\n\\n(I mean no disrespect by that)\\n\\n\\nThe last GOP administration is responsible for 4,000+ dead American service people, 2,800 dead Americans in NYC and 100,000+ dead Iraqi\'s.\\n\\n\\nAnd you\'rre going to get your panties in a bunch over 4 dead Americans killed in a terrorist attack in the ME -  because you think you can make some political hay over it?\\n\\n\\nWith all due respect - phucque you and the horse you rode in on", "disqusTypePrev": "approved", "object": {"link": "http://www.politico.com/story/2014/07/benghazi-trey-gowdy-108625.html#comment-1473078627", "id": "tag:gnip.disqus.com:2012:comment/1hae9j", "disqusId": "1473078627", "objectType": "comment"}, "disqusType": "approved", "verb": "update", "gnip": {"language": {"value": "en"}}, "id": "tag:gnip.disqus.com:2012:comment/1hae9j/update/2014-07-07T19:33:59/ab61a0faa8752910cf949d274bfd51d1090b2dedc63bf7fe5d21a608441f6ac3", "target": {"website": {"id": "tag:gnip.disqus.com:2012:forum/itsy", "disqusId": "1422873"}, "link": "http://www.politico.com/story/2014/07/benghazi-trey-gowdy-108625.html", "disqusId": "2825453635", "postedTime": "2014-07-07T21:59:23+00:00", "id": "tag:gnip.disqus.com:2012:thread/a69my0m", "objectType": "article"}, "disqusMessageTimestamp": "2014-07-07T23:34:26+00:00", "actor": {"preferredUsername": "disqus_xs2HXcqTks", "id": "tag:gnip.disqus.com:2012:account/ani2ch", "disqusId": "95458297", "objectType": "person"}, "postedTime": "2014-07-07T23:33:59+00:00"}"""
        
        o = self.objs["struct"]

        # these 5 values should have data 
        for i, record in o.file_reader( json_string=GOOD_STRUCT_STRING ):
            record_string = o.procRecord(record)
            [ self.assertNotEqual( "None", x ) 
                for x in record_string.split(self.delim)[-self.struct_length:] 
            ] 
        
        # these 2 values should not have data (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_STRUCT_STRING ):
            record_string = o.procRecord(record)
            [ self.assertEqual( "None", x ) 
                for x in record_string.split(self.delim)[-self.struct_length+3:] 
            ] 

    def test_status_fields(self):
        """
        Test for the presence of non-"None" values in a good record when using the 'struct' 
        option, and "None"s in an intentionally-damaged record.
        """ 
        
        # this activity (chosen from data/) has all the struct fields in it  
        GOOD_STATUS_STRING = VALID_ACTIVITY

        # remove the "inReplyTo" key in this activity, so that the last two options_struct fields should be "None" 
        BAD_STATUS_STRING = """{"body": ">\\"incompetent situations like Benghazi.\\"<\\n\\n\\nBlow it out yourass, Goober\\n\\n\\n(I mean no disrespect by that)\\n\\n\\nThe last GOP administration is responsible for 4,000+ dead American service people, 2,800 dead Americans in NYC and 100,000+ dead Iraqi\'s.\\n\\n\\nAnd you\'rre going to get your panties in a bunch over 4 dead Americans killed in a terrorist attack in the ME -  because you think you can make some political hay over it?\\n\\n\\nWith all due respect - phucque you and the horse you rode in on", "inReplyTo": {"author": {"id": "tag:gnip.disqus.com:2012:account/a8m86y", "disqusId": "64154961"}, "id": "tag:gnip.disqus.com:2012:comment/1haa42", "disqusId": "1473071832", "objectType": "comment"}, "object": {"link": "http://www.politico.com/story/2014/07/benghazi-trey-gowdy-108625.html#comment-1473078627", "id": "tag:gnip.disqus.com:2012:comment/1hae9j", "disqusId": "1473078627", "objectType": "comment"}, "verb": "update", "disqusMessageTimestamp": "2014-07-07T23:34:26+00:00", "id": "tag:gnip.disqus.com:2012:comment/1hae9j/update/2014-07-07T19:33:59/ab61a0faa8752910cf949d274bfd51d1090b2dedc63bf7fe5d21a608441f6ac3", "target": {"website": {"id": "tag:gnip.disqus.com:2012:forum/itsy", "disqusId": "1422873"}, "link": "http://www.politico.com/story/2014/07/benghazi-trey-gowdy-108625.html", "disqusId": "2825453635", "postedTime": "2014-07-07T21:59:23+00:00", "id": "tag:gnip.disqus.com:2012:thread/a69my0m", "objectType": "article"}, "gnip": {"language": {"value": "en"}}, "actor": {"preferredUsername": "disqus_xs2HXcqTks", "id": "tag:gnip.disqus.com:2012:account/ani2ch", "disqusId": "95458297", "objectType": "person"}, "postedTime": "2014-07-07T23:33:59+00:00"}"""
        
        o = self.objs["status"]

        # these 3 values should have data 
        for i, record in o.file_reader( json_string=GOOD_STATUS_STRING ):
            record_string = o.procRecord(record)
            [ self.assertNotEqual( "None", x ) 
                for x in record_string.split(self.delim)[-self.status_length:] 
            ] 
        
        # these 2 values should not have data (should have "None" instead) 
        for i, record in o.file_reader( json_string=BAD_STATUS_STRING ):
            record_string = o.procRecord(record)
            [ self.assertEqual( "None", x ) 
                for x in record_string.split(self.delim)[-self.status_length+1:] 
            ]
    
    
    def test_all_length(self):
        """
        Check the number of fields being output. (Update the expected results when new 
        features are added to the module.)
        """
        # grab the right instance 
        o = self.objs["all"]
        
        # ensure our file_reader has worked correctly 
        g = o.file_reader( json_string=VALID_ACTIVITY )
        self.assertIsInstance( g.next(), tuple ) 

        # use sample record above 
        for i, record in o.file_reader( json_string=VALID_ACTIVITY ):
            loop = True
            record_string = o.procRecord(record)
            # should have 3 extra fields now 
            self.assertEqual( len( record_string.split( self.delim ) )
                                , self.all_length 
                            )



 
if __name__ == "__main__":
    unittest.main()
